Package: StepReg
Version: 1.6.7
Title: A Comprehensive and Intuitive R Package for Stepwise Regression
        Analysis
Authors@R: c(
    person("Junhui", "Li", , "junhui.li11@umassmed.edu", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0003-3973-1700")),
    person("Kai", "Hu", , "kai.hu@umassmed.edu", role = c("aut")),
    person("Xiaohuan", "Lu", , role = c("aut")),
    person("Sushmita", "N Nayak", , role = c("ctb", "aut")),
    person("Cesar", "Bautista Sotelo", , role = c("ctb", "aut")),
    person("Michael", "A Lodato", , role = c("ctb", "aut")),
    person("Wenxin", "Liu", , role = c("aut")),
    person("Lihua", "Julie Zhu", , role = c("aut"))
  )
Date: 2026-08-31
Description: Stepwise regression is a statistical technique used for model selection. This package streamlines stepwise regression analysis by supporting multiple regression types(linear, Cox, logistic, Poisson, Gamma, and negative binomial), incorporating popular selection strategies(forward, backward, bidirectional, and subset), and offering essential metrics. It enables users to apply multiple selection strategies and metrics in a single function call, visualize variable selection processes, and export results in various formats. StepReg offers a data-splitting option to address potential issues with invalid statistical inference and a randomized forward selection option to avoid overfitting. We validated StepReg's accuracy using public datasets within the SAS software environment. For an interactive web interface, users can install the companion 'StepRegShiny' package. The methodology is described in Li et al. (2026) <doi:10.32614/RJ-2026-005>.
License: MIT + file LICENSE
URL: https://journal.r-project.org/articles/RJ-2026-005/,
        https://github.com/JunhuiLi1017/StepReg
BugReports: https://github.com/JunhuiLi1017/StepReg/issues
VignetteBuilder: knitr
Suggests: knitr, testthat, BiocStyle, kableExtra, cowplot, rmarkdown
Imports: ggplot2, ggrepel, MASS, survival, flextable, pROC, survAUC
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Repository: CRAN
Packaged: 2026-08-31 13:26:53 UTC; lij11
Author: Junhui Li [cre, aut] (ORCID: <https://orcid.org/0000-0003-3973-1700>),
  Kai Hu [aut],
  Xiaohuan Lu [aut],
  Sushmita N Nayak [ctb, aut],
  Cesar Bautista Sotelo [ctb, aut],
  Michael A Lodato [ctb, aut],
  Wenxin Liu [aut],
  Lihua Julie Zhu [aut]
Maintainer: Junhui Li <junhui.li11@umassmed.edu>
Date/Publication: 2026-08-31 19:31:03 UTC
Built: R 4.6.1; ; 2026-08-31 22:56:49 UTC; unix

Package: StepReg
Version: 1.6.6
Title: Stepwise Regression Analysis
Authors@R: c(
    person("Junhui", "Li", , "junhui.li11@umassmed.edu", role = c("cre"),
           comment = c(ORCID = "0000-0003-3973-1700")),
    person("Junhui", "Li", , "junhui.li11@umassmed.edu", role = c("aut")),
    person("Kai", "Hu", , "kai.hu@umassmed.edu", role = c("aut")),
    person("Xiaohuan", "Lu", , role = c("aut")),
    person("Sushmita", "N Nayak", , role = c("ctb")),
    person("Cesar", "Bautista Sotelo", , role = c("ctb")),
    person("Michael", "A Lodato", , role = c("ctb")),
    person("Wenxin", "Liu", , role = c("aut")),
    person("Lihua", "Julie Zhu", , role = c("aut"))
  )
Date: 2026-06-03
Description: Stepwise regression is a statistical technique used for model selection. This package streamlines stepwise regression analysis by supporting multiple regression types(linear, Cox, logistic, Poisson, Gamma, and negative binomial), incorporating popular selection strategies(forward, backward, bidirectional, and subset), and offering essential metrics. It enables users to apply multiple selection strategies and metrics in a single function call, visualize variable selection processes, and export results in various formats. StepReg offers a data-splitting option to address potential issues with invalid statistical inference and a randomized forward selection option to avoid overfitting. We validated StepReg's accuracy using public datasets within the SAS software environment. For an interactive web interface, users can install the companion 'StepRegShiny' package.
License: MIT + file LICENSE
BugReports: https://github.com/JunhuiLi1017/StepReg/issues
VignetteBuilder: knitr
Suggests: knitr, testthat, BiocStyle, kableExtra, cowplot, rmarkdown
Imports: ggplot2, ggrepel, MASS, survival, flextable, pROC, survAUC
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Repository: CRAN
Packaged: 2026-06-04 15:15:59 UTC; lij11
Author: Junhui Li [cre] (ORCID: <https://orcid.org/0000-0003-3973-1700>),
  Junhui Li [aut],
  Kai Hu [aut],
  Xiaohuan Lu [aut],
  Sushmita N Nayak [ctb],
  Cesar Bautista Sotelo [ctb],
  Michael A Lodato [ctb],
  Wenxin Liu [aut],
  Lihua Julie Zhu [aut]
Maintainer: Junhui Li <junhui.li11@umassmed.edu>
Date/Publication: 2026-06-04 16:00:03 UTC
Built: R 4.6.0; ; 2026-06-04 19:56:55 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,StepReg)
S3method(print,StepReg)
export(performance)
export(report)
export(stepwise)
import(ggplot2)
importFrom(MASS,glm.nb)
importFrom(flextable,align)
importFrom(flextable,autofit)
importFrom(flextable,flextable)
importFrom(flextable,save_as_docx)
importFrom(flextable,save_as_html)
importFrom(flextable,save_as_pptx)
importFrom(flextable,save_as_rtf)
importFrom(ggrepel,geom_label_repel)
importFrom(pROC,auc)
importFrom(pROC,roc)
importFrom(stats,anova)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,pf)
importFrom(stats,predict)
importFrom(stats,reformulate)
importFrom(stats,sigma)
importFrom(stats,terms)
importFrom(survAUC,AUC.hc)
importFrom(survAUC,AUC.sh)
importFrom(survAUC,AUC.uno)
importFrom(survival,Surv)
importFrom(survival,concordance)
importFrom(survival,coxph)
importFrom(utils,combn)

# StepReg <a href="https://github.com/JunhuiLi1017/StepReg"><img src="man/figures/logo.png" align="right" height="138" /></a>
---
- #### An R package for stepwise regression analysis
---

StepReg is an R package that streamlines stepwise regression analysis by supporting multiple regression types, incorporating popular selection strategies, and offering essential metrics.

## Key Features

- **Multiple Regression Types**: Linear, logistic, Cox, Poisson, Gamma, and negative binomial regression
- **Selection Strategies**: Forward selection, backward elimination, bidirectional elimination, and best subsets
- **Selection Metrics**: AIC, AICc, BIC, CP, HQ, adjRsq, SL, SBC, IC(3/2), IC(1)
- **Advanced Features**: 
  - Strata variables for Cox regression
  - Continuous-nested-within-class effects
  - multivariable multiple linear stepwise regression
- **Multicollinearity Detection**: Automatic detection and handling of multicollinearity
- **Visualization**: Plot functions for variable selection processes
- **Reporting**: Export results in various formats (HTML, DOCX, XLSX, PPTX)
- **Shiny App**: Interactive web interface for non-programmers

## Installation

### Install from CRAN
```r
pak::pkg_install("StepReg")
```

or

```r
install.packages("StepReg")
```

### Or install from GitHub
```r
devtools::install_github("JunhuiLi1017/StepReg")
```

## Quick Start

```r
library(StepReg)

# Basic linear regression
data(mtcars)
formula <- mpg ~ .
res <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = "bidirection",
  metric = "AIC"
)

# View results
res
summary(res$bidirection$AIC)
```

## Advanced Features

### Strata Variables in Cox Regression

```r
library(survival)
data(lung)
lung$sex <- factor(lung$sex)

# Cox regression with strata
formula <- Surv(time, status) ~ age + sex + ph.ecog + strata(inst)
res <- stepwise(
  formula = formula,
  data = lung,
  type = "cox",
  strategy = "forward",
  metric = "AIC"
)
```

### Continuous-Nested-Within-Class Effects

```r
data(mtcars)
mtcars$am <- factor(mtcars$am)

# Nested effects
formula <- mpg ~ am + wt:am + disp:am + hp:am
res <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = "bidirection",
  metric = "AIC"
)
```

## Documentation

- [Vignette](https://CRAN.R-project.org/package=StepReg) - Comprehensive guide with examples
- [Reference Manual](https://CRAN.R-project.org/package=StepReg) - Function documentation

## Shiny Application

- [StepReg](https://junhuili1017.shinyapps.io/StepReg/) - StepReg Shiny Appliction

## Important Note

StepReg should **NOT** be used for statistical inference unless the variable selection process is explicitly accounted for, as it can compromise the validity of the results. This limitation does not apply when StepReg is used for prediction purposes.

## Citation

If you use StepReg in your research, please cite:

```r
citation("StepReg")
```

## Questions?
Please raise an issue [here](https://github.com/JunhuiLi1017/StepReg/issues/new).

## ----include=FALSE------------------------------------------------------------
knitr::opts_chunk$set(comment = NA)

## ----message = FALSE----------------------------------------------------------
library(StepReg)
data(mtcars)
formula <- mpg ~ .
res <- stepwise(formula = formula,
                data = mtcars,
                type = "linear",
                include = c("qsec"),
                strategy = "bidirection",
                metric = c("AIC"))

## ----message = FALSE----------------------------------------------------------
res

## ----message = FALSE----------------------------------------------------------
summary(res$bidirection$AIC)

## ----message = FALSE----------------------------------------------------------
plot(res, strategy = "bidirection", process = "overview")
plot(res, strategy = "bidirection", process = "detail")

## ----eval = FALSE-------------------------------------------------------------
# report(res, report_name = "path_to/demo_res", format = "html")

## ----echo = FALSE-------------------------------------------------------------
library(knitr)
library(kableExtra)

Regression <- c("linear", "logit", "cox", "poisson", "gamma")
Reponse <- c("continuous", "binary", "time-to-event", "count", "continuous and positively skewed")
df <- data.frame(Regression, Reponse)

kable(df, format = "html", caption = 'Common regression categories') %>% kable_styling()

## ----echo = FALSE-------------------------------------------------------------
Strategy <- c("Forward Selection", "Backward Elimination", "Bidirectional Elimination", "Best Subsets")
Description <- c("In forward selection, the algorithm starts with an empty model (no predictors) and adds in variables one by one. Each step tests the addition of every possible predictor by calculating a pre-selected metric. Add the variable (if any) whose inclusion leads to the most statistically significant fit improvement. Repeat this process until more predictors no longer lead to a statistically better fit.",
                "In backward elimination, the algorithm starts with a full model (all predictors) and deletes variables one by one. Each step test the deletion of every possible predictor by calculating a pre-selected metric. Delete the variable (if any) whose loss leads to the most statistically significant fit improvement. Repeat this process until less predictors no longer lead to a statistically better fit.",
                "Bidirectional elimination is essentially a forward selection procedure combined with backward elimination at each iteration. Each iteration starts with a forward selection step that adds in predictors, followed by a round of backward elimination that removes predictors. Repeat this process until no more predictors are added or excluded.",
                "Stepwise algorithms add or delete one predictor at a time and output a single model without evaluating all candidates. Therefore, it is a relatively simple procedure that only produces one model. In contrast, the *Best Subsets* algorithm calculates all possible models and output the best-fitting models with one predictor, two predictors, etc., for users to choose from.")
df <- data.frame(Strategy, Description)

kable(df, format = "html", caption = 'Model selection strategy') %>% kable_styling()

## ----echo = FALSE-------------------------------------------------------------
Statistic <- c(
"${n}$",
"${p}$",
"${q}$",
"$\\sigma^2$",
"${SST}$",
"${SSE}$",
"$\\text{LL}$",
"${|  |}$",
"$\\ln()$")

Meanings <- c(
"Sample Size",
"Number of parameters including the intercept",
"Number of dependent variables",
"Estimate of pure error variance from fitting the full model",
"Total sum of squares corrected for the mean for the dependent variable, which is a numeric value for UMR and a matrix for multivariate regression",
"Error sum of squares, which is a numeric value for UMR and a matrix for multivariate regression",
"The natural logarithm of likelihood",
"The determinant function",
"The natural logarithm")

kable_styling(kable(data.frame(Statistic,Meanings),format = "html", align='l', escape = F, caption = 'Statistics in selection metric'))

## ----echo = FALSE-------------------------------------------------------------
Abbreviation <- c("", "AIC", "AICc", "BIC", "Cp", "HQ", "IC(1)", "IC(3/2)", "SBC", "SL", "adjRsq")
Definition <- c("",
                "Akaike’s Information Criterion",
                "Corrected Akaike’s Information Criterion",
                "Sawa Bayesian Information Criterion",
                "Mallows’ Cp statistic",
                "Hannan and Quinn Information Criterion",
                "Information Criterion with Penalty Coefficient Set to 1",
                "Information Criterion with Penalty Coefficient Set to 3/2",
                "Schwarz Bayesian Information Criterion",
                "Significance Level (pvalue)",
                "Adjusted R-square statistic")

Formula_in_Linear <- c("linear",
                       "$n\\ln\\left(\\frac{|\\text{SSE}|}{n}\\right) + 2pq + n + q(q+1)$ <br>[@Hurvich_Tsai_1989; @Al-Subaihi_2002]$^1$",
                       "$n\\ln\\left(\\frac{|\\text{SSE}|}{n}\\right) + \\frac{nq(n+p)}{n-p-q-1}$ <br>[@Hurvich_Tsai_1989; @Bedrick_Tsai_1994]$^2$",
                       "$n\\ln\\left(\\frac{SSE}{n}\\right) + 2(p+2)o - 2o^2, o = \\frac{n\\sigma^2}{SSE}$ <br>[@Sawa_1978; @Judge_1985] <br>not available for MMR",
                       "$\\frac{SSE}{\\sigma^2} + 2p - n$ <br> [@Mallows_1973; @Hocking_1976] <br>not available for MMR",
                       "$n\\ln\\left(\\frac{|\\text{SSE}|}{n}\\right) + 2pq\\ln(\\ln(n))$ <br>[@Hannan_Quinn_1979; @McQuarrie_Tsai_1998; @Hurvich_Tsai_1989]",
                       "$n\\ln\\left(\\frac{|\\text{SSE}|}{n}\\right) + p$ <br>[@Nelder_Wedderburn_1972; @Smith_Spiegelhalter_1980] not available for MMR",
                       "$n\\ln\\left(\\frac{|\\text{SSE}|}{n}\\right) + \\frac{3}{2}p$ <br>[@Smith_Spiegelhalter_1980] <br>not available for MMR",
                       "$n\\ln\\left(\\frac{|\\text{SSE}|}{n}\\right) + pq \\ln(n)$ <br>[@Hurvich_Tsai_1989; @Schwarz_1978; @Judge_1985; @Al-Subaihi_2002] <br>not available for MMR",
                       "$\\textit{F test}$ for UMR and $\\textit{Approximate F test}$ for MMR",
                       "$1 - \\frac{(n-1)(1-R^2)}{n-p}$, <br> where $R^2=1 - \\frac{SSE}{SST}$ <br>[@Darlington_1968; @Judge_1985] <br>not available for MMR")

Formula_in_Logit_Cox_Poisson_Gamma <- c("logit, cox, poisson and gamma",
                                        "$-2\\text{LL} + 2p$ <br>[@Darlington_1968; @Judge_1985]",
                                        "$-2\\text{LL} + \\frac{n(n+p)}{n-p-2}$ <br>[@Hurvich_Tsai_1989]",
                                        "not available",
                                        "not available",
                                        "$-2\\text{LL} + 2p\\ln(\\ln(n))$ <br>[@Hannan_Quinn_1979]",
                                        "$-2\\text{LL} + p$ <br>[@Nelder_Wedderburn_1972; @Smith_Spiegelhalter_1980]",
                                        "$-2\\text{LL} + \\frac{3}{2}p$ <br>[@Smith_Spiegelhalter_1980]",
                                        "$-2\\text{LL} + p\\ln(n)$ <br>[@Schwarz_1978; @Judge_1985]",
                                        "Forward: LRT and Rao Chi-square test (logit, poisson, gamma); LRT (cox); <br><br>Backward: Wald test",
                                        "not available")
df <- data.frame(Abbreviation, Definition, Formula_in_Linear, Formula_in_Logit_Cox_Poisson_Gamma)
colnames(df) <- c("Abbreviation","Definition","Formula","")

kable(df, format = "html", align = "l", 
      booktabs = TRUE, escape = F, 
      caption = 'Abbreviation, Definition, and Formula of the Selection Metric for Linear, Logit, Cox, Possion, and Gamma regression') %>%
  footnote(number = c("Unsupported AIC formula (which does not affect the selection process as it only differs by constant additive and multiplicative factors):\n
                      $AIC=n\\ln\\left(\\frac{SSE}{n}\\right) + 2p$ [@Darlington_1968; @Judge_1985]", 
                      "Unsupported AICc formula (which does not affect the selection process as it only differs by constant additive and multiplicative factors):\n
                      $AICc=\\ln\\left(\\frac{SSE}{n}\\right) + 1 + \\frac{2(p+1)}{n-p-2}$ [@McQuarrie_Tsai_1998]")) %>%
  kable_styling() %>%
  column_spec(3, width = "0.5in") %>%
  column_spec(4, width = "0.4in")

## ----eval = FALSE-------------------------------------------------------------
# formula  =  Surv(time, status) ~ . + strata(inst)

## ----eval = FALSE-------------------------------------------------------------
# mtcars$am <- as.factor(mtcars$am)
# formula <- mpg ~ am + cyl:am + wt:am + disp:am + hp:am + qsec:am + vs:am + gear:am + carb:am

## ----eval = FALSE-------------------------------------------------------------
# formula <- cbind(mpg, drat) ~ .

## ----echo = FALSE-------------------------------------------------------------
Syntax <- c("`y ~ x1 + x2`", "`y ~ .`", "`y ~ . - x1`", "`y ~ x1 * x2`", "`y ~ x1:x2`", "`cbind(y1, y2) ~ .`", "`y ~ . + 0` or `y ~ . - 1`", "`Surv(time, status) ~ . + strata(strata_var)`")
Description <- c("Multiple predictors", "All variables in dataset", "All variables except x1", "Main effects and interaction", "Continuous-nested-within-class effects", "Multiple response variables", "No intercept", "Cox regression with strata")
Example <- c("`mpg ~ cyl + wt`", "`mpg ~ .`", "`mpg ~ . - disp`", "`mpg ~ cyl * am`", "`mpg ~ cyl:am`", "`cbind(mpg, drat) ~ .`", "`mpg ~ . + 0`", "`Surv(time, status) ~ age + sex + strata(inst)`")

df <- data.frame(Syntax, Description, Example)
kable(df, format = "html", caption = 'Formula syntax supported by StepReg') %>% kable_styling()

## ----message = FALSE----------------------------------------------------------
data(mtcars)
## make sure the categorical variable is a factor variable
mtcars$am <- as.factor(mtcars$am)
str(mtcars)

formula <- mpg ~ am + cyl:am + disp:am + am:hp + drat:am + wt:am + qsec:am + vs:am + gear:am + carb:am
res1 <- stepwise(formula = formula,
                 data = mtcars,
                 type = "linear",
                 include = c("cyl:am", "am"),
                 strategy = "forward",
                 metric = "AIC",
                 test_ratio = 0.2)
res1

## ----message = FALSE----------------------------------------------------------
summary(res1$forward$AIC)

## ----message = FALSE----------------------------------------------------------
performance(res1)

## ----plot_res1, warning = FALSE-----------------------------------------------
plot_list <- list()
plot_list[["forward"]][["detail"]] <- plot(res1, process = "detail")
plot_list[["forward"]][["overview"]] <- plot(res1, process = "overview")
cowplot::plot_grid(plotlist = plot_list$forward, ncol = 1)

## ----eval = FALSE-------------------------------------------------------------
# formula <- mpg ~ . + 0

## ----eval = FALSE-------------------------------------------------------------
# formula <- mpg ~ . - 1

## ----eval = FALSE-------------------------------------------------------------
# formula <- mpg ~ cyl + disp + hp + wt + vs + am + 0

## ----eval = FALSE-------------------------------------------------------------
# formula <- mpg ~ . - 1 - disp - wt

## ----message = FALSE, warning = FALSE-----------------------------------------
formula <- mpg ~ .
res2 <- stepwise(formula = formula,
                 data = mtcars,
                 type = "linear",
                 strategy = c("forward", "backward"),
                 metric = c("AIC", "BIC", "SL"),
                 sle = 0.05,
                 sls = 0.05,
                 test_ratio = 0.3)
res2

## ----message = FALSE, warning = FALSE, fig.width=9, fig.height=12-------------
plot_list <- setNames(
  lapply(c("forward", "backward"),function(i){
    setNames(
      lapply(c("detail","overview"),function(j){
        plot(res2,strategy=i,process=j)
    }),
    c("detail","overview")
    )
  }),
  c("forward", "backward")
)

cowplot::plot_grid(plotlist = plot_list$forward, ncol = 1, rel_heights = c(2, 1))
cowplot::plot_grid(plotlist = plot_list$backward, ncol = 1, rel_heights = c(2, 1))

## ----message = FALSE----------------------------------------------------------
summary(res2$forward$SL)

## ----message = FALSE----------------------------------------------------------
performance(res2)

## ----eval = FALSE-------------------------------------------------------------
# report(res2, report_name = "results", format = c("html", "docx"))

## -----------------------------------------------------------------------------
formula <- cbind(mpg, drat) ~ . + 0
res3 <- stepwise(formula = formula,
                 data = mtcars,
                 type = "linear",
                 strategy = "bidirection",
                 metric = c("AIC", "HQ"),
                 test_ratio=0.2,
                 feature_ratio = 0.9)
res3

plot_list <- setNames(
  lapply(c("bidirection"),function(i){
    setNames(
      lapply(c("detail","overview"),function(j){
        plot(res3,strategy=i,process=j)
    }),
    c("detail","overview")
    )
  }),
  c("bidirection")
)

cowplot::plot_grid(plotlist = plot_list$bidirection, ncol = 1, rel_heights = c(2, 1))

## ----message = FALSE----------------------------------------------------------
summary(res3$bidirection$AIC)

## ----message = FALSE----------------------------------------------------------
performance(res3)

## ----message = FALSE----------------------------------------------------------
data(remission)
str(remission)

formula <- remiss ~ .
res4 <- stepwise(formula = formula,
                 data = remission,
                 type = "logit",
                 include= "cell",
                 strategy = "forward",
                 metric = "AIC",
                 test_ratio = 0.2)
res4

plot_list <- setNames(
  lapply(c("forward"),function(i){
    setNames(
      lapply(c("detail","overview"),function(j){
        plot(res4,strategy=i,process=j)
    }),
    c("detail","overview")
    )
  }),
  c("forward")
)
cowplot::plot_grid(plotlist = plot_list$forward, ncol = 1, rel_heights = c(2, 1))

## ----message = FALSE----------------------------------------------------------
summary(res4$forward$AIC)

## ----message = FALSE----------------------------------------------------------
performance(res4)

## ----message = FALSE----------------------------------------------------------
formula <- remiss ~ . + 0
res5 <- stepwise(formula = formula,
                  data = remission,
                  type = "logit",
                  strategy = "subset",
                  metric = "SBC",
                  best_n = 3,
                  test_ratio = 0.2)
res5

plot_list <- setNames(
  lapply(c("subset"),function(i){
    setNames(
      lapply(c("detail","overview"),function(j){
        plot(res5,strategy=i,process=j)
    }),
    c("detail","overview")
    )
  }),
  c("subset")
)
cowplot::plot_grid(plotlist = plot_list$subset, ncol = 1, rel_heights = c(2, 1))

## ----message = FALSE----------------------------------------------------------
summary(res5$subset$SBC)

## ----message = FALSE----------------------------------------------------------
performance(res5)

## ----message = FALSE----------------------------------------------------------
data(lung)
library(survival)
lung <- na.omit(lung)
lung$sex <- factor(lung$sex, levels = c(1, 2))
str(lung)

formula  =  Surv(time, status) ~ . + strata(sex)
res6 <- stepwise(formula = formula,
                 data = lung,
                 type = "cox",
                 strategy = "forward",
                 metric = c("AICc", "SL"),
                 sle = 0.1,
                 test_ratio = 0.2)
res6

plot_list <- setNames(
  lapply(c("forward"),function(i){
    setNames(
      lapply(c("detail","overview"),function(j){
        plot(res6,strategy=i,process=j)
    }),
    c("detail","overview")
    )
  }),
  c("forward")
)
cowplot::plot_grid(plotlist = plot_list$forward, ncol = 1, rel_heights = c(2, 1))

## ----message = FALSE----------------------------------------------------------
summary(res6$forward$AICc)
summary(res6$forward$SL)

## ----message = FALSE----------------------------------------------------------
performance(res6)

## ----message = FALSE----------------------------------------------------------
data(creditCard)
str(creditCard)

formula  = reports ~ .
res7 <- stepwise(formula = formula,
                 data = creditCard,
                 type = "poisson",
                 strategy = "forward",
                 metric = "SL",
                 sle = 0.05,
                 test_ratio = 0.2)
res7

## ----message = FALSE----------------------------------------------------------
plot_list <- setNames(
  lapply(c("forward"),function(i){
    setNames(
      lapply(c("detail","overview"),function(j){
        plot(res7,strategy=i,process=j)
      }),
      c("detail","overview")
    )
  }),
  c("forward")
)
cowplot::plot_grid(plotlist = plot_list$forward, ncol = 1, rel_heights = c(2, 1))

## ----message = FALSE----------------------------------------------------------
summary(res7$forward$SL)

## ----message = FALSE----------------------------------------------------------
performance(res7)

## ----eval = FALSE-------------------------------------------------------------
# library(StepRegShiny)
# StepRegGUI()

## ----sessionInfo, echo = FALSE------------------------------------------------
sessionInfo()

#' Leukemia Remission Dataset
#'
#' A dataset containing information about leukemia remission and associated risk factors.
#' This dataset is commonly used for demonstrating logistic regression analysis in medical research.
#'
#' @format A data frame with 27 observations and 7 variables:
#' \describe{
#'   \item{remiss}{Binary outcome variable indicating leukemia remission status:
#'     \itemize{
#'       \item 1 = Remission occurred
#'       \item 0 = No remission
#'     }
#'   }
#'   \item{cell}{Numeric. Cellularity of the marrow clot section (percentage)}
#'   \item{smear}{Numeric. Smear differential percentage of blasts}
#'   \item{infil}{Numeric. Percentage of absolute marrow leukemia cell infiltrate}
#'   \item{li}{Numeric. Percentage labeling index of the bone marrow leukemia cells}
#'   \item{blast}{Numeric. Absolute number of blasts in the peripheral blood}
#'   \item{temp}{Numeric. Highest temperature (in Fahrenheit) before treatment}
#' }
#'
#' @details This dataset is particularly useful for:
#' \itemize{
#'   \item Demonstrating logistic regression analysis
#'   \item Studying risk factors for leukemia remission
#'   \item Teaching medical statistics and predictive modeling
#' }
#'
#' @source Lee, E. T. (1974). "A Computer Program for Linear Logistic Regression Analysis."
#' Computer Programs in Biomedicine 4:80–92.
#'
#' @references
#' \itemize{
#'   \item Lee, E. T. (1974). "A Computer Program for Linear Logistic Regression Analysis."
#'   Computer Programs in Biomedicine 4:80–92.
#'   \item Penn State University Statistics Online. "Logistic Regression Example."
#'   \url{https://online.stat.psu.edu/stat501/book/export/html/1011}
#' }
#'
#' @examples
#' \dontrun{
#' # Load the dataset
#' data(remission)
#' 
#' # View first few rows
#' head(remission)
#' 
#' # Summary statistics
#' summary(remission)
#' 
#' # Run logistic regression
#' model <- glm(remiss ~ ., data = remission, family = binomial)
#' summary(model)
#' }
#'
#' @usage data(remission)
#' @name remission
#' @docType data
#' @keywords datasets
"remission"

#' Credit Card Application Dataset
#'
#' A dataset containing credit history information for credit card applicants. This dataset is sourced from the AER package.
#'
#' @format A data frame with 1,319 observations and 12 variables:
#' \describe{
#'   \item{card}{Factor. Whether the credit card application was accepted (Yes/No)}
#'   \item{reports}{Numeric. Number of major derogatory reports on the applicant's credit history}
#'   \item{age}{Numeric. Age in years plus twelfths of a year (e.g., 30.5 represents 30 years and 6 months)}
#'   \item{income}{Numeric. Annual income in USD (divided by 10,000)}
#'   \item{share}{Numeric. Ratio of monthly credit card expenditure to yearly income}
#'   \item{expenditure}{Numeric. Average monthly credit card expenditure in USD}
#'   \item{owner}{Factor. Home ownership status (Yes/No)}
#'   \item{selfemp}{Factor. Self-employment status (Yes/No)}
#'   \item{dependents}{Numeric. Number of dependents}
#'   \item{months}{Numeric. Number of months living at current address}
#'   \item{majorcards}{Numeric. Number of major credit cards held}
#'   \item{active}{Numeric. Number of active credit accounts}
#' }
#'
#' @details This dataset is commonly used for credit risk analysis and modeling credit card approval decisions. 
#' It provides a comprehensive view of various factors that may influence credit card application outcomes.
#'
#' @source Greene, W.H. (2003). Econometric Analysis, 5th edition. Upper Saddle River, NJ: Prentice Hall.
#'
#' @seealso \code{\link[AER]{CreditCard}} for the original dataset in the AER package
#'
#' @examples
#' data(creditCard)
#' summary(creditCard)
#' @usage data(creditCard)
#' @name creditCard
#' @docType data
#' @keywords datasets
"creditCard"

#' Print Stepwise Regression Results
#'
#' Displays the final model fit statistics from a StepReg object. This function provides
#' a concise summary of the selected model's performance metrics.
#'
#' @param x A StepReg object containing the results of stepwise regression analysis.
#' 
#' @param ... Additional arguments passed to the print method (currently not used).
#'
#' @return Invisibly returns the printed object. The function displays:
#'   \itemize{
#'     \item Final model fit statistics for each strategy and metric combination
#'   }
#' 
#' @details The print method provides a focused view of the final model's performance,
#' showing the selected variables and their corresponding fit statistics. This is useful
#' for quickly assessing the model's quality without the detailed step-by-step selection
#' process (which can be viewed using \code{\link{stepwise}}).
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data(mtcars)
#' 
#' # Run stepwise regression
#' formula <- mpg ~ .
#' result <- stepwise(
#'   formula = formula,
#'   data = mtcars,
#'   type = "linear",
#'   strategy = "forward",
#'   metric = "AIC"
#' )
#' 
#' # Print final model statistics
#' result
#' }
#'
#' @seealso \code{\link{stepwise}} for creating StepReg objects
#' @seealso \code{\link{plot.StepReg}} for visualization of results
print.StepReg <- function(x, ...){
  nohid <- attr(x, "nonhidden")
  print(x[nohid])
}

#' NCCTG Lung Cancer Data
#'
#' Survival in patients with advanced lung cancer from the North Central Cancer Treatment Group. Performance scores rate how well the patient can perform usual daily activities.
#'
#' 
#' @note This dataset is sourced from the survival package.
#' @seealso \code{\link[survival]{lung}} for the original dataset in the survival package
#' 
#' @format A data frame containing 228 observations on 10 variables.
#' \describe{
#'   \item{inst}{Institution code}
#'   \item{time}{Survival time in days}
#'   \item{status}{censoring status 1=censored, 2=dead}
#'   \item{age}{Age in years}
#'   \item{sex}{Male=1 Female=2}
#'   \item{ph.ecog}{ECOG performance score as rated by the physician. 0=asymptomatic, 1= symptomatic but completely ambulatory, 2= in bed < 50\% of the day, 3= in bed > 50\% of the day but not bedbound, 4 = bedbound}
#'   \item{ph.karno}{Karnofsky performance score (bad=0-good=100) rated by physician}
#'   \item{pat.karno}{Karnofsky performance score as rated by patient}
#'   \item{meal.cal}{Calories consumed at meals}
#'   \item{wt.loss}{Weight loss in last six months (pounds)}
#' }
#'
#'
#' @source Terry Therneau. (2021). survival: Survival Analysis. R package version 3.4-2. \url{https://CRAN.R-project.org/package=survival}
#'
#' 
#' @references 
#' 
#' \itemize{
#'   \item Loprinzi CL. Laurie JA. Wieand HS. Krook JE. Novotny PJ. Kugler JW. Bartel J. Law M. Bateman M. Klatt NE. et al. Prospective evaluation of prognostic variables from patient-completed questionnaires. North Central Cancer Treatment Group. Journal of Clinical Oncology. 12(3):601-7, 1994.
#' }
#' 
#' @examples
#' data(lung)
#' summary(lung)
#' @usage data(lung)
#' @name lung
#' @docType data
#' @keywords datasets
NULL

# Stepwise helper functions
# 
# @author Junhui Li, Kai Hu, Xiaohuan Lu

match_multiple_args <- function(value, choice){
	value <- sapply(value, function(x) match.arg(x, choice))
	names(value) <- NULL
	value
}

getXname <- function(formula, data) {
	term_form <- terms(formula, data = data)
	x_name <- attr(term_form, "term.labels")
	strata_check <- grepl("strata\\(", x_name)
	if(any(strata_check)) {
		strata_var <- gsub("strata\\((.*)\\)", "\\1", x_name[strata_check])
		x_name <- x_name[!strata_check]
		x_name <- x_name[!x_name %in% strata_var]
	}
	continuous_check <- grepl(":", x_name)
	if(any(continuous_check)) {
		cont_class_var <- x_name[continuous_check]
		back_var <- gsub(".*:", "", cont_class_var)
		front_var <- gsub(":.*", "", cont_class_var)
		## make sure front_var and back_var is a factor variable at least
		if(!all(sapply(data[, back_var], is.factor) | sapply(data[, front_var], is.factor))) {
			index <- !(sapply(data[, back_var], is.factor) | sapply(data[, front_var], is.factor))
			stop("For continuous-nested-within-class variable ",cont_class_var[index],", make sure one of the variables is a factor variable!")
		}
	}
	return(x_name)
}

getYname <- function(formula, data) {
	term_form <- terms(formula, data = data)
	vars <- as.character(attr(term_form, "variables"))[ -1 ]
	y_name <- vars[attr(term_form, "response")]
	return(y_name)
}

dummyNameList <- function(x_name, data){
  factor_vars <- names(which(sapply(data, is.factor)))
  overlap_var_list <- sapply(factor_vars, function(i) {
    paste0(i, levels(data[[i]]))
  })
  overlap_check <- sapply(overlap_var_list, function(i){
    if(any(i %in% x_name)){
      org_x_name <- x_name[x_name %in% i]
      stop(paste0("there is a conflict for variable name: ",org_x_name," and dummy name: ", i, "\n"))
    }
  })
  return(overlap_var_list)
}

getIntercept <- function(formula, data, type) {
	term_form <- terms(formula, data = data)
	x_name <- attr(term_form, "term.labels")
	if(type != 'cox') {
		if(attr(term_form, "intercept") == 0) {
			intercept <- "0"
		}else{
			intercept <- "1"
		}
	}else{ # for 'cox', we need to set intercept to NULL if no strata() in formula
		if(any(grepl("strata\\(", x_name))) {
			intercept <- x_name[grepl("strata\\(", x_name)]
		}else{
			intercept <- '0'
		}
	}
	return(intercept)
}

getMergedVar <- function(include) {
	# obtain a single string concatenating all include variables by space
	if(is.character(include)) {
		merge_include_name <- paste0(include, collapse = " ")
	}else if(is.null(include)) {
		merge_include_name <- "NULL"
	}
	return(merge_include_name)
}

getModel <- function(data, type, intercept, x_name, y_name, weight, method = c("efron", "breslow", "exact")) {
	formula_raw <- reformulate(c(intercept, x_name), y_name)
	if(type == 'linear') {
		## cannot perform multivariate multiple regression in glm() function
		#lm_raw <- glm(formula_raw, data = data, weights = weights, family = "gaussian")
		model_raw <- do.call("lm", list(formula_raw, data = quote(data), weights = weight))
	}else if(type == "logit") {
		model_raw <- do.call("glm", list(formula_raw, data = quote(data), weights = weight, family = "binomial"))
	}else if(type == "poisson") {
		model_raw <- do.call("glm", list(formula_raw, data = quote(data), weights = weight, family = "poisson"))
	}else if(type == "gamma") {
		model_raw <- do.call("glm", list(formula_raw, data = quote(data), weights = weight, family = "Gamma"))
	}else if(type == 'cox') {
		## "method" is only used for cox regression
		method <- match.arg(method)
		model_raw <- do.call("coxph", list(formula_raw, data = quote(data), weights = weight, method = method))
	}else if(type == "negbin") {
		model_raw <- do.call("glm.nb", list(formula_raw, data = quote(data), weights = weight))
	}
	return(model_raw)
}

getSigmaFullModel <- function(lmf, type, n_y) {
	if(type == "linear") {
		if(lmf$rank >= nrow(as.data.frame(lmf$residuals))) {
			sigma_value <- 0
		}else{ 
			sigma_value <- sum(deviance(lmf)/df.residual(lmf))/n_y
		}
	}else{
		sigma_value <- 0
	}
	return(sigma_value)
}

getMulticolX <- function(data, x_name, tolerance) {
	## expand the continuous-nested-within-class variable to two variables
	continuous_check <- grepl(":", x_name)
	if(any(continuous_check)) {
		cont_class_var <- x_name[continuous_check]
		front_var <- unique(gsub(":.*", "", cont_class_var))
		back_var <- unique(gsub(".*:", "", cont_class_var))
		x_col_name <- c(x_name, front_var, back_var)
	}else{
		x_col_name <- x_name
	}
	x_name_var <- x_col_name[x_col_name %in% colnames(data)]
	x_matrix <- as.matrix(data[, x_name_var])
	x_matrix_num <- apply(x_matrix, 2, function(x) {
		if(is.character(x)) as.numeric(as.factor(x)) else as.numeric(x)
	})
	qrx_list <- qr(x_matrix_num, tol = tolerance)
	rank0 <- qrx_list$rank
	pivot0 <- qrx_list$pivot
	if(rank0 < length(pivot0)) {
		multico_x <- colnames(qrx_list$qr)[pivot0[(rank0	+	1) : length(pivot0)]]
	multico_x <- x_name[x_col_name %in% multico_x]
	}else{
		multico_x <- "NULL"
	}
	return(multico_x) # return multicollineared x_names
}

getTestMethod <- function(data, model_raw, type, metric, n_y, test_method_linear, test_method_glm, test_method_cox) {
	if(type == "linear") {
		n_obs <- nrow(data)
		# get sigma for BIC and CP
		if(n_y == 1) {
			test_method <- "F"
		}else{
			test_method <- test_method_linear
		}
	}else if(type == "logit" | type == "poisson" | type == "gamma" | type == "negbin") {
		test_method <- test_method_glm
	}else if(type == "cox") {
		test_method <- test_method_cox
	}
	return(test_method)
}

# For "subset", "forward", "backward", "bidirection": each model needs to calculate PIC. 
# Fit Model Statistics
#
# Fit Model Statistics with least square or likelihood method to return an information criteria value 
#
# @param metric Information criteria, including AIC, AICc, BIC, CP, HQ, HQc, adjRsq and SBC
# 
# @param fit Object of linear model or general linear model
# 
# @param type "linear", "cox", "logit", "poisson", "gamma" and "negbin": to calculate information criteria value; for "linear", the "Least Square" method will be used; for others, "Maximum Likelyhood" method will be used.
# 
# @param sigma_value Sigma value for calculation of 'BIC' and 'CP'

getModelFitStat <- function(metric = c("AIC", "AICc", "BIC", "CP", "HQ", "HQc", "adjRsq", "SBC", "IC(3/2)", "IC(1)"), fit, type = c("linear", "logit", "poisson", "cox", "gamma", "negbin"), sigma_value) {
	# "LeastSquare" is for linear; "Likelihood" is for cox and logit; cox and logit are essentially the same except for sample size calculation.
	if (type == "linear") {
		resMatrix <- as.matrix(fit$residuals)
		SSEmatrix <- t(resMatrix) %*% resMatrix
		SSE <- abs(det(SSEmatrix))
		p <- fit$rank
		n <- nrow(resMatrix)
		#yName <- rownames(attr(fit$terms, "factors"))[1]
		vars <- as.character(attr(fit$terms, "variables"))[ -1 ]
		yName <- vars[attr(fit$terms, "response")]
		Y <- as.matrix(fit$model[, yName])
		nY <- ncol(Y)
		if(metric == "AIC") {
			PIC <- n*log(SSE/n) + 2*p*nY + nY*(nY + 1) + n
		}else if(metric == "AICc") {
			PIC <- n*log(SSE/n) + n*(n + p)*nY/(n - p - nY - 1)
		}else if(metric == "CP") {
			PIC <- SSE/sigma_value + 2*p - n
		}else if(metric == "HQ") {
			#PIC <- n*log(SSE/n) + 2*log(log(n))*p*nY/n
			PIC <- n*log(SSE/n) + 2*log(log(n))*p*nY
		}else if(metric == "HQc") {
			#PIC <- n*log(SSE*SSE/n) + 2*log(log(n))*p*nY/(n - p - nY - 1)
			PIC <- n*log(SSE/n) + 2*log(log(n))*p*nY*n/(n - p - nY - 1)
		}else if(metric == "IC(1)") {
			PIC <- n*log(SSE/n) + p*nY + nY*(nY + 1) + n
		}else if(metric == "IC(3/2)") {
			PIC <- n*log(SSE/n) + 1.5*p*nY + nY*(nY + 1) + n
		}else if(metric == "BIC") {
			PIC <- n*log(SSE/n) + 2*(2 + p)*(n*sigma_value/SSE) - 2*(n*sigma_value/SSE)*(n*sigma_value/SSE)
		#}#else if(metric == "Rsq") {
			#PIC <- 1 - (SSE/SST)
			#PIC <- summary(fit)$r.squared
		}else if(metric == "adjRsq") {
			#PIC <- 1 - (SSE/SST)*(n - 1)/(n - p)
			PIC <- summary(fit)$adj.r.squared
		}else if(metric == "SBC") {
			PIC <- n*log(SSE/n) + log(n)*p*nY
		}
	} else if (type %in% c("logit", "poisson", "cox", "gamma", "negbin")) {
		ll <- logLik(fit)[1]
		p <- attr(logLik(fit), "df")
		if (type == "cox") {
			n <- fit$nevent
		} else {
			n <- nrow(fit$data)
		}
		if(metric == "IC(1)") {
			PIC <- -2*ll + p
		}else if(metric == "IC(3/2)") {
			PIC <- -2*ll + 1.5*p
		}else if(metric == "SBC") {
			PIC <- -2*ll + p*log(n)
		}else if(metric == "AICc") {
			#PIC <- -2*ll + 2*p*(p + 1)/(n - p - 1)
			PIC <- -2*ll + n*(n + p)/(n - p - 2)
		}else if(metric == "AIC") {
			PIC <- -2*ll + 2*p
		}else if(metric == "HQ") {
			PIC <- -2*ll + 2*p*log(log(n))
		}else if(metric == "HQc") {
			PIC <- -2*ll + 2*p*n*log(log(n))/(n - p - 2)
		}
	}
	return(PIC)
}

getInitialSubSet <- function(data, type, metric, y_name, intercept, include, weight, test_method, sigma_value) {
	# obtain the initial model information: if no include variable, return NULL, otherwise return a matrix containing columns of "NumberOfVariables", metric, and "VariablesInModel"
	# metric refers to PIC method, e.g. AIC, BIC, etc. for "logit" and "cox" type, if metric is "SL", the PIC is calculated differently
	initial_process_table <- NULL
	if (length(include) != 0) {
		initial_process_table <- data.frame(NumberOfVariables=numeric(),
																				metric=numeric(),
																				VariablesInModel=character())
		colnames(initial_process_table) <- c("NumberOfVariables", metric, "VariablesInModel")
		x_fit <- getModel(data = data, type = type, intercept = intercept, x_name = c(intercept, include), y_name = y_name, weight = weight, method = test_method)

		if(metric == "SL") {
			if(type == "cox") {
				pic_set <- x_fit$score
			}else {
				fit_reduce <- switch(type,
														 "logit"	 = glm(reformulate(intercept, y_name), data = data, weights = weight, family = "binomial"),
														 "poisson" = glm(reformulate(intercept, y_name), data = data, weights = weight, family = "poisson"),
														 "gamma"	 = glm(reformulate(intercept, y_name), data = data, weights = weight, family = "Gamma"),
														 "negbin"	= glm.nb(reformulate(intercept, y_name), data = data, weights = weight),
														 "linear"	= lm(reformulate(intercept, y_name), data = data, weights = weight)
														 
				)
				f_pic_vec <- getAnovaStat(add_or_remove = "add", include = include, fit_reduced = fit_reduce, fit_full = x_fit, type = type, test_method = test_method)
				pic_set <- f_pic_vec[1]
			}
		}else{
			pic_set <- getModelFitStat(metric, x_fit, type, sigma_value)
		}
		initial_process_table[1, 1:3] <- c(as.numeric(intercept) + length(include), pic_set, paste(c(intercept, include), collapse = " "))
	}
	return(initial_process_table)
}

getFinalSubSet <- function(data, type, metric, x_notin_model, initial_process_table, y_name, include, weight, intercept, best_n = Inf, test_method, sigma_value) {
	process_table <- initial_process_table
	#nv=8
	for (nv in 1:length(x_notin_model)) {
		com_table <- as.data.frame(combn(x_notin_model, nv))
		n_test <- ncol(com_table)
		com_var <- apply(com_table, 2, paste, collapse = " ")
		sub_process_table <- matrix(rep(c(nv + length(include) + as.numeric(intercept), NA), each = n_test), n_test, 2)
		com_var_df <- cbind(paste(c(intercept, include), collapse = " "), data.frame(com_var))
		com_var_set <- apply(com_var_df, 1, paste, collapse = " ")
		sub_process_table <- data.frame(sub_process_table, c(com_var_set))
		colnames(sub_process_table) <- c("NumberOfVariables", metric, "VariablesInModel")
		colnames(com_table) <- com_var_set
		x_test_list <- as.list(com_table)
		x_name_list <- lapply(x_test_list, function(x) {c(intercept, include, x)})
		x_fit_list <- lapply(x_name_list, function(x) {getModel(data = data, type = type, intercept = intercept, x_name = x, y_name = y_name, weight = weight, method = test_method)})
		
		if(metric == "SL") {
			if(type == "cox") {
				pic_set <- sapply(x_fit_list, function(x) {x$score})
			}else {
				fit_reduce <- switch(type,
														 "logit"	 = glm(reformulate(intercept, y_name), data = data, weights = weight, family = "binomial"),
														 "poisson" = glm(reformulate(intercept, y_name), data = data, weights = weight, family = "poisson"),
														 "gamma"	 = glm(reformulate(intercept, y_name), data = data, weights = weight, family = "Gamma"),
														 "negbin"	= glm.nb(reformulate(intercept, y_name), data = data, weights = weight),
														 "linear"	= lm(reformulate(intercept, y_name), data = data, weights = weight)
				)
				f_pic_vec <- sapply(x_fit_list, function(x) {getAnovaStat(add_or_remove = "add", include = include, fit_reduced = fit_reduce, fit_full = x, type = type, test_method = test_method)})
				pic_set <- f_pic_vec[1, ]
			}
		}else{
			pic_set <- sapply(x_fit_list, function(x) {getModelFitStat(metric, x, type, sigma_value)})
		}
		sub_process_table[, 2] <- pic_set
		
		if (metric %in% c("SL", "adjRsq")) {
			# "Rsq" and "adjRsq" are for type "linear"; "SL" is for type "logit" and "cox"
			decreasing = TRUE
		} else{
			decreasing = FALSE
		}
		sub_process_table_sort <- sub_process_table[order(sub_process_table[, 2], decreasing = decreasing), ]
		if(nrow(sub_process_table_sort) < best_n) {
			best_n_model <- nrow(sub_process_table_sort)
		}else{
			best_n_model <- best_n
		}
		process_table <- rbind(process_table, sub_process_table_sort[1:best_n_model, ])
	}
	return(process_table)
}

getSelectedXName <- function(process_table, metric) {
	if (metric %in% c("SL", "adjRsq")) {
		# "Rsq" and "adjRsq" are for type "linear"; "SL" is for type "logit" and "cox"
		x_name_selected <- unlist(strsplit(process_table[which.max(as.numeric(process_table[, metric])), 3], " "))
	} else{
		x_name_selected <- unlist(strsplit(process_table[which.min(as.numeric(process_table[, metric])), 3], " "))
	}
	x_name_selected <- x_name_selected[!x_name_selected %in% ""]
	return(x_name_selected)
}

getSubsetWrapper <- function(data, type, metric, x_name, y_name, intercept, include, weight, best_n, test_method, sigma_value) {
	# a wrapper to obtain x_name_selected
	## obtain initial model info
	initial_process_table <- getInitialSubSet(data, type, metric, y_name, intercept, include, weight = weight, test_method, sigma_value)
	
	## obtain final model info
	x_notin_model <- setdiff(x_name, include)
	process_table <- getFinalSubSet(data, type, metric, x_notin_model, initial_process_table, y_name, include, weight = weight, intercept, best_n, test_method, sigma_value)
	
	## add rownames to sort the variables in process_table when output
	rownames(process_table) <- c(1:nrow(process_table))
	return(process_table)
}

formatTable <- function(tbl, tbl_name = "Test") {
	tbl_list <- list(tbl)
	names(tbl_list) <- tbl_name
	class(tbl_list) <- "StepReg"
	return(tbl_list)
}

getTable1SummaryOfParameters <- function(formula, data, type, x_name, y_name, merged_multico_x,merged_include, strategy, metric, sle, sls, test_method, tolerance, intercept, test_ratio, feature_ratio, seed) {
	# generate: table1: Summary of Parameters
	table_1_summary_of_parameters <- data.frame(
		Parameter = c(  "initial formula",
						"regression type",
						"selection strategy", 
						"stepwise metric", 
						"significance level for entry (sle)", 
						"significance level for stay (sls)",
						"included variable",
						"test method", 
						"tolerance of multicollinearity", 
						"multicollinearity variable", 
						"intercept",
						"test ratio",
						"feature ratio",
						"seed"),
		Value = c(paste0(deparse(formula), collapse = ""),
							type,
							paste0(strategy, collapse=" & "), 
							paste0(metric, collapse=" & "), 
							sle, 
							sls, 
							merged_include, 
							test_method, 
							tolerance, 
							merged_multico_x, 
							intercept,
							test_ratio,
							feature_ratio,
							seed)
	)
	if(type == 'cox') {
		# "intercept" is not relevant
		table_1_summary_of_parameters <- table_1_summary_of_parameters[-11, ]
	}
	# get rid of unrelevant variables from table 1:
	nrow_sls <- which(table_1_summary_of_parameters$Parameter %in% "significance level for stay (sls)")
	nrow_sle <- which(table_1_summary_of_parameters$Parameter %in% "significance level for entry (sle)")
	nrow_test_method <- which(table_1_summary_of_parameters$Parameter %in% "test method")
	if(any(metric %in% "SL")) {
		if(any(strategy == "bidirection") | (any(strategy == "forward") & any(strategy == "backward"))) {
			table_1_summary_of_parameters <- table_1_summary_of_parameters
		}else if(any(strategy == "forward") & (!any(strategy == "bidirection") & !any(strategy == "backward"))) {
			table_1_summary_of_parameters <- table_1_summary_of_parameters[-nrow_sls, ]
		}else if(any(strategy == "backward") & (!any(strategy == "bidirection") & !any(strategy == "forward"))) {
			table_1_summary_of_parameters <- table_1_summary_of_parameters[-nrow_sle, ]
		}else{
			table_1_summary_of_parameters <- table_1_summary_of_parameters[-c(nrow_sle,nrow_sls), ]
		}
	}else if(!any(metric %in% "SL") & type != 'cox') {
		table_1_summary_of_parameters <- table_1_summary_of_parameters[-c(nrow_sls,nrow_sle,nrow_test_method), ]
	}
	return(table_1_summary_of_parameters)
}

getTable2TypeOfVariables <- function(model) {
	x_name <- getXname(formula(model), model.frame(model))
	y_name <- getYname(formula(model), model.frame(model))
	
	class_var <- attr(model$terms, "dataClasses")
	class_table <- matrix(c(names(class_var), class_var), ncol=2)
	table2_class_table <- cbind(class_table,NA)
	table2_class_table[class_table[, 1] %in% y_name, 3] <- "Dependent"
	table2_class_table[!class_table[, 1] %in% y_name, 3] <- "Independent"
	colnames(table2_class_table) <- c("Variable_name", "Variable_class", "Variable_type")
	rownames(table2_class_table) <- NULL
	return(as.data.frame(table2_class_table))
}

#note1: test_method_linear should be 'F' for univariate and c(“Pillai”, “Wilks”, “Hotelling-Lawley”, “Roy”) for multivariates, so cant use summary() to get p value for multivariates.
#note2: for cox regression, no matter what the parameter is in test="x" of anova(), always return the same p value. 
# 1) Wald test p value can only obtained from summary(). 
# 2) cannot select 'Rao' or 'LRT' in anova for effect entered.
#note3: for logit/poison/gamma regression, 
#1) Wald test p value obtained from summary(), since we dont know if test='chisq' in anova is for wald test. 
#2) can select 'Rao' or 'LRT' in anova for effect entered.
#3) for factor effect in formula, xname1(for example sex) in formula is different with ones(for example sex1 and sex2) in summary(), we need to keep the min p value of paste("set",c(1:2)), and then rename it with 'sex'.
#getAnovaStat(fit_reduced = x_fit_list[[1]], fit_full = fit_x_in_model, type = type, test_method = test_method)
getAnovaStat <- function(add_or_remove = "add", intercept, include, fit_reduced, fit_full, type, test_method) {
	if (type == "linear") {
		ptype <- 'Pr(>F)'
		if(test_method %in% c("Pillai", "Wilks", "Hotelling-Lawley", "Roy")) {
			stattype <- 'approx F'
		}else{
			stattype <- 'F'
		}
	} else if (type == "logit" | type == "poisson" | type == "gamma" | type == "negbin") {
		if(add_or_remove == "add") {
			if(test_method == "Rao") {
				stattype <- "Rao"
			}else if(test_method == "LRT") {
				stattype <- "Deviance"
			}
			ptype <- 'Pr(>Chi)'
		}else{
			stattype <- "z value"
			ptype <- 'Pr(>|z|)'
		}
	} else if (type == "cox") {
		# need to update for wald test
		if(add_or_remove == "add") {
			test_method <- ""
			ptype <- c('P(>|Chi|)', 'Pr(>|Chi|)')
			stattype <- "Chisq"
		}else{
			stattype <- "z"
			ptype <- 'Pr(>|z|)'
		}
	}
	if(type != "linear" & add_or_remove == "remove") { #wald test for logit and cox with add_or_remove = "remove"
		stat_table <- coef(summary(fit_full))
		stat_table[, stattype] <- stat_table[, stattype]^2
		coef_name <- rownames(stat_table)
		ptype <- colnames(stat_table)[colnames(stat_table) %in% ptype]
		xlevels <- fit_full$xlevels
		
		if(intercept == "1") { # remove intercept for scanning all other variables' p value
			if(nrow(stat_table) == 2) {
				stat_table <- stat_table[-1, , drop = FALSE]
			} else {
				stat_table <- stat_table[-1,]
			}
		}
		if(length(xlevels) == 0) {
			stat_table <- as.data.frame(stat_table)
			stat_table$Var <- rownames(stat_table)
		} else {
			coef_name_levels <- unlist(lapply(names(xlevels), function(i) {
				paste(i,xlevels[[i]],sep="")
			}))
			names(coef_name_levels) <- rep(names(xlevels),sapply(xlevels,length))
			#coef_name_include <- c(coef_name_levels[names(coef_name_levels) %in% include], include)
			#stat_table_no_include <- stat_table[which(!coef_name %in% coef_name_include), , drop = FALSE]
			#coef_name_parent <- sapply(rownames(stat_table_no_include), function(i){
			coef_name_parent <- sapply(rownames(stat_table), function(i) {
				if(i %in% coef_name_levels) {
					names(coef_name_levels)[coef_name_levels %in% i]
				} else {
					i
				}
			})
			stat_table <- as.data.frame(stat_table)
			stat_table$Var <- coef_name_parent
		}
		
		pic_set <- stat_table[, ptype]
		#names(pic_set) <- rownames(stat_table)
		names(pic_set) <- stat_table$Var
		statistics <- stat_table[, stattype]
		#names(statistics) <- rownames(stat_table)
		names(statistics) <- stat_table$Var
		#maxPVar <- rownames(stat_table)[which.max(pic_set)]
		pic <- list(pic_set[!names(pic_set) %in% include])
		statistics <- list(statistics[!names(statistics) %in% include])
		# maxPVar <- names(which.max(pic_set))
		# statistics <- stat_table[maxPVar, stattype]
		# pic <- stat_table[maxPVar, ptype]
	} else {
		stat_table <- anova(fit_reduced, fit_full, test = test_method)
		ptype <- names(stat_table)[names(stat_table) %in% ptype]
		statistics <- stat_table[2, stattype]
		pic <- stat_table[2, ptype]
	}
	return(c("statistics" = statistics, "pic" = pic))
}

## get pic based on model fit, it needs fit_reduced and fit_full for SL and only fit_formula for other metrics
## used in stepwise in step0: SL:0, 1, inf	non - SL:
## fit_fm<- fit_intercept
getInitStepModelStat <- function(fit_intercept, fit_fm, type, strategy, metric, intercept, include, test_method, sigma_value) {
	if(metric == "SL") {
		if(!is.null(include)) {
			if(all(include %in% attr(fit_fm$terms, "term.labels")) & strategy != "backward") {
				#add_or_remove, intercept, fit_reduced, fit_full, type, test_method
				f_pic_vec <- getAnovaStat(add_or_remove = "add", include = include, fit_reduced = fit_intercept, fit_full = fit_fm, type = type, test_method = test_method)
				pic <- f_pic_vec["pic"]
			}else{
				pic <- 1
			}
		}else{
			pic <- 1
		}
	}else{
		if(strategy == "backward") {
			pic <- getModelFitStat(metric, fit_fm, type, sigma_value)
		}else{
			if(!is.null(include)) {
				if(all(include %in% attr(fit_fm$terms, "term.labels"))) {
					pic <- getModelFitStat(metric, fit_fm, type, sigma_value)
				}else{
					if(intercept == '1') {
						pic <- getModelFitStat(metric, fit_fm, type, sigma_value)
					}else{
						if(metric %in% c("adjRsq") & type == "linear") {
							pic <- 0
						}else{
							pic <- Inf
						}
					}
				}
			}else{
				if(intercept == '1') {
					pic <- getModelFitStat(metric, fit_fm, type, sigma_value)
				}else{
					if(metric %in% c("adjRsq") & type == "linear") {
						pic <- 0
					}else{
						pic <- Inf
					}
				}
			}
		}
	}
	return(pic)
}

#return 3 num for IC and remove 1st of 3 num for SL
getNumberEffect <- function(fit, type) {
	if(type == "linear") {
		vec <- c(length(attr(fit$terms, "term.labels")) + attr(fit$terms,"intercept"), fit$rank)
	}else if(type == "logit" | type == "poisson" | type == "gamma" | type == "negbin") {
		vec <- c(fit$rank, fit$rank)
	}else if(type == "cox") {
		vec <- c(attr(logLik(fit), "df"), attr(logLik(fit), "df"))
	}
	return(vec)
}

initialProcessTable <- function(metric) {
	sub_init_process_table <- data.frame(Step = numeric(), 
															 EffectEntered = character(), 
															 EffectRemoved = character(), 
															 NumberEffect = numeric(), 
															 NumberParams = numeric(), 
															 metric = numeric())
	colnames(sub_init_process_table)[ncol(sub_init_process_table)] <- metric
	#colnames(sub_init_process_table)[ncol(sub_init_process_table)] <- ifelse(metric == "SL", "PValue", metric)
	return(sub_init_process_table)
}

getInitialStepwise <- function(data, type, strategy, metric, intercept, include, x_name, y_name, weight, test_method, sigma_value) {
	sub_init_process_table <- initialProcessTable(metric)
	process_table <- sub_init_process_table
	if(strategy == "backward") {
		add_or_remove <- "remove"
		#the order of variable in x_in_model will affect pic calculation.
		x_in_model <- c(setdiff(x_name, include))
		x_notin_model <- NULL
		fit_full <- getModel(data = data, type = type, intercept = intercept, x_name = c(include, x_in_model), y_name = y_name, weight = weight, method = test_method)
		pic <- getInitStepModelStat(fit_intercept = NULL, fit_fm = fit_full, type = type, strategy = strategy, metric = metric, intercept = intercept, include = include, test_method = test_method, sigma_value)
		num_eff_para_in <- getNumberEffect(fit = fit_full, type = type)
		process_table[1, ] <- c(rep("", 3), num_eff_para_in, pic)
	}else{
		add_or_remove <- "add"
		x_in_model <- NULL
		x_notin_model <- setdiff(x_name, include)
		#x_notin_model <- x_notin_model[sample(1:length(x_notin_model), size = round(feature_ratio * length(x_notin_model), 0))]
		## for intercept
		fit_intercept <- getModel(data = data, type = type, intercept = intercept, x_name = NULL, y_name = y_name, weight = weight, method = test_method)
		pic <- getInitStepModelStat(fit_intercept = fit_intercept, fit_fm = fit_intercept, type = type, strategy = strategy, metric = metric, intercept = intercept, include = include, test_method = test_method, sigma_value)
		num_eff_para_in <- getNumberEffect(fit = fit_intercept, type = type)
		process_table[1, ] <- c("", intercept, "", num_eff_para_in, pic)
		## for include
		if(!is.null(include)) {
			fit_include <- getModel(data = data, type = type, intercept = intercept, x_name = include, y_name = y_name, weight = weight, method = test_method)
			pic <- getInitStepModelStat(fit_intercept = fit_intercept, fit_fm = fit_include, type = type, strategy = strategy, metric = metric, intercept = intercept, include = include, test_method = test_method, sigma_value)
			num_eff_para_in <- getNumberEffect(fit = fit_include, type = type)
			#sub_init_process_table[1, ] <- c("", paste0(include, collapse = " "), "", abs(anova(fit_include, fit_intercept)[2, 'Df']), num_eff_para_in[-1], pic)
			sub_init_process_table[1, ] <- c("", paste0(include, collapse = " "), "", num_eff_para_in, pic)
			process_table <- rbind(process_table, sub_init_process_table)
		}
	}
	process_table$Step <- c(1:nrow(process_table))
	return(list("add_or_remove" = add_or_remove, "x_in_model" = x_in_model, "x_notin_model" = x_notin_model, "process_table" = process_table))
}

getCandStepModel <- function(add_or_remove, data, type, metric, weight, y_name, x_in_model, x_notin_model, intercept, include, test_method, sigma_value, feature_ratio, dummy_name_list) {
	fit_x_in_model <- getModel(data = data, type = type, intercept = intercept, x_name = c(include, x_in_model), y_name = y_name, weight = weight, method = test_method)
	BREAK <- FALSE
	if(add_or_remove == "add") {
		x_test <- x_notin_model[sample(1:length(x_notin_model), size = round(feature_ratio * length(x_notin_model), 0))]
	}else{
		x_test <- x_in_model
	}
	if(length(x_test) == 0) {
		BREAK <- TRUE
	}
	x_test_list <- as.list(x_test)
	names(x_test_list) <- x_test

	if(BREAK == FALSE) {
		if(add_or_remove == "add") {
			x_name_list <- lapply(x_test_list, function(x) {c(x_in_model, x)})
		}else{
			x_name_list <- lapply(x_test_list, function(x) {setdiff(x_in_model, x)})
		}
		x_fit_list <- lapply(x_name_list, function(x) {getModel(data = data, type = type, intercept = intercept, x_name = c(include, x), y_name = y_name, weight = weight, method = test_method)})
		
		if(metric == "SL") {
			if(type != "linear" & add_or_remove == "remove") {
				f_pic_vec <- getAnovaStat(add_or_remove = "remove", intercept = intercept, include = include, fit_full = fit_x_in_model, type = type, test_method = test_method)
				pic_set <- f_pic_vec[[2]]
				f_set <- f_pic_vec[[1]]
				# var_set <- f_pic_vec[3]
				# names(pic_set) <- var_set
				# names(f_set) <- var_set
			}else{
				f_pic_vec <- sapply(x_fit_list, function(x) {getAnovaStat(add_or_remove = add_or_remove, include = include, fit_reduced = x, fit_full = fit_x_in_model, type = type, test_method = test_method)})
				pic_set <- f_pic_vec[2, ]
				f_set <- f_pic_vec[1, ]
				names(pic_set) <- colnames(f_pic_vec)
				names(f_set) <- colnames(f_pic_vec)
			}
		} else {
			if(add_or_remove == "remove" & length(x_test) == 1 & intercept == "0") {
				if(metric == 'adjRsq'){
					pic_set <- 0
				} else {
					pic_set <- Inf
				}
				names(pic_set) <- x_test
			}else{
				pic_set <- sapply(x_fit_list, function(x) {getModelFitStat(metric, x, type, sigma_value)})
			}
		}
		if(metric == "adjRsq" | (metric == "SL" & add_or_remove == "remove")) {
			pic <- max(pic_set)
			minmax_var <- names(which.max(pic_set))

			if(length(dummy_name_list) > 0) {
				for(i in 1:length(dummy_name_list)) {
					if(minmax_var %in% dummy_name_list[[i]]) {
						minmax_var <- names(dummy_name_list)[i]
					}
				}
			}

			best_candidate_model <- x_fit_list[[minmax_var]]
		} else {
			pic <- min(pic_set)
			minmax_var <- names(which.min(pic_set))
			best_candidate_model <- x_fit_list[[minmax_var]]
			if(sum(pic_set %in% pic) > 1 & metric == "SL") {
				Fvalue <- max(f_set)
				minmax_var <- names(which.max(f_set))
				best_candidate_model <- x_fit_list[[minmax_var]]
				pic <- pic_set[minmax_var]
			}
		}
		if(type != "cox") {
			if(best_candidate_model$rank == fit_x_in_model$rank & add_or_remove == "add") {
				BREAK <- TRUE
			}
		}
		return(list("pic" = pic, "minmax_var" = minmax_var, "best_candidate_model" = best_candidate_model, "BREAK" = BREAK, "pic_list" = pic_set))
	}else{
		return(list("BREAK" = BREAK))
	}
}

getGoodnessFit <- function(best_candidate_model, y_name, metric) {
	smr <- summary(best_candidate_model)
	n_y <- ncol(as.matrix(best_candidate_model$model[, y_name]))
	if(n_y == 1) {
		f <- smr$fstatistic
		t <- smr
		if(is.nan(f[1])) {
			pval <- NaN
		}else{
			pval <- pf(f[1], f[2], f[3], lower.tail = F)
		}
	}else{
		for(ny in 1:n_y) {
			f <- smr[[ny]]$fstatistic
			if(is.nan(f[1])) {
				pval <- NaN
			}else{
				pval <- pf(f[1], f[2], f[3], lower.tail = F)
			}
		}
	}
	if(is.nan(pval) == TRUE & (metric != 'adjRsq')) {
		BREAK <- TRUE
	}else{
		BREAK <- FALSE
	}
	return(BREAK)
}

checkEnterOrRemove <- function(add_or_remove, best_candidate_model, type, metric, sle, sls, y_name, pic, process_table) {
	if(metric == 'SL') {
		if(add_or_remove == "remove") {
			indicator <- pic > sls
		}else{
			indicator <- pic < sle
		}
	}else if(metric == 'adjRsq') {
		indicator <- pic > as.numeric(process_table[nrow(process_table), 6])
	}else{
		indicator <- pic <= as.numeric(process_table[nrow(process_table), 6])
	}
	if(indicator == TRUE & type == "linear" & (metric != "adjRsq")) {
		if(best_candidate_model$rank != 0) {
			BREAK <- getGoodnessFit(best_candidate_model, y_name, metric)
		}
	}else{
		BREAK <- FALSE
	}
	return(c("indicator" = indicator, "BREAK" = BREAK))
}

updateXinModel <- function(add_or_remove, indicator, best_candidate_model, type, metric, BREAK, pic, x_in_model, x_notin_model, process_table, minmax_var, pic_list) {
	sub_init_process_table <- initialProcessTable(metric)
	if(indicator == TRUE & BREAK == FALSE) {
		if(add_or_remove == "add") {
			x_in_model <- append(x_in_model, minmax_var)
			x_notin_model <- setdiff(x_notin_model, minmax_var)
			sub_init_process_table[1, ] <- c(as.numeric(process_table[nrow(process_table), 1]) + 1, minmax_var, "", getNumberEffect(fit = best_candidate_model, type), pic)
		}else{
			x_notin_model <- append(x_notin_model, minmax_var)
			x_in_model <- setdiff(x_in_model, minmax_var)
			sub_init_process_table[1, ] <- c(as.numeric(process_table[nrow(process_table), 1]) + 1, "", minmax_var, getNumberEffect(fit = best_candidate_model, type), pic)
		}
		process_table <- rbind(process_table, sub_init_process_table)
		#process_table[nrow(process_table), 1] <- as.numeric(process_table[1, nrow(process_table) - 1])	+	1
		pic_set <- unlist(pic_list)
	}else{
		BREAK <- TRUE
		pic_set <- NULL
	}
	return(list("BREAK" = BREAK, "process_table" = process_table, "x_in_model" = x_in_model, "x_notin_model" = x_notin_model, "pic_set" = pic_set))
}

getFinalStepModel <- function(add_or_remove, data, type, strategy, metric, sle, sls, weight, y_name, x_in_model, x_notin_model, intercept, include, process_table, test_method, sigma_value, feature_ratio, dummy_name_list) {
	pic_df <- NULL
	while(TRUE) {
		out_cand_stepwise <- getCandStepModel(add_or_remove, data, type, metric, weight = weight, y_name, x_in_model, x_notin_model, intercept, include, test_method, sigma_value, feature_ratio, dummy_name_list)
		BREAK <- out_cand_stepwise$BREAK
		minmax_var <- out_cand_stepwise$minmax_var
		if(BREAK == TRUE) {
			break
		}
		best_candidate_model <- out_cand_stepwise$best_candidate_model
		pic <- out_cand_stepwise$pic
		pic_list <- list(sort(out_cand_stepwise$pic_list))
		
		out_check <- checkEnterOrRemove(add_or_remove, best_candidate_model, type, metric, sle, sls, y_name, pic, process_table)
		indicator <- out_check[1]
		BREAK <- out_check["BREAK"]
		if(BREAK == TRUE) {
			break
		}
		
		out_updateX <- updateXinModel(add_or_remove, indicator, best_candidate_model, type, metric, BREAK, pic, x_in_model, x_notin_model, process_table, minmax_var, pic_list)
		x_in_model <- out_updateX$x_in_model
		x_notin_model <- out_updateX$x_notin_model
		process_table <- out_updateX$process_table
		pic_set <- out_updateX$pic_set
		if(!is.null(pic_set)) {
			pic_df <- rbind(pic_df,data.frame(strategy, metric, step = process_table[nrow(process_table),1], "variable" = names(pic_set), "value" = pic_set))
		}
		rownames(pic_df) <- NULL
		# stop stepwise infinite loops
		# enter remove
		# x1
		#			 x1(stop here)
		#			 
		# enter remove		 
		#			 x1(stop here)
		# x1
		if(nrow(process_table) > 1) {
			last2_step <- process_table[nrow(process_table) -1 , ]
			last1_step <- process_table[nrow(process_table), ]
			if(last2_step[, 2] != "" & last2_step[, 2] == last1_step[, 3]) {
				break
			}else if(last2_step[, 3] == last1_step[, 2] & last1_step[, 2] != "") {
				process_table <- process_table[ - nrow(process_table), ]
				break
			}
		}
		
		if(indicator == TRUE) {
			if(strategy == 'bidirection') {
				if(add_or_remove == "remove") {
					next
				}else{
					add_or_remove <- "remove"
					next
				}
			}else{
				next
			}
		}else{
			if(strategy == 'bidirection' & add_or_remove == "remove") {
				add_or_remove <- "add"
				next
			}else{
				break
			}
		}
	}
	
	if(type == "cox" && strategy != "backward") {
		process_table <- process_table[-1, ]
		process_table$Step <- as.numeric(process_table$Step) - 1
	}
	return(list("process_table" = process_table, "x_in_model" = x_in_model, "x_notin_model" = x_notin_model, "pic_df" = pic_df))
}

getStepwiseWrapper <- function(data, type, strategy, metric, sle, sls, weight, x_name, y_name, intercept, include, test_method, sigma_value, feature_ratio, dummy_name_list) {
	#fit_full <- getModel(data = data, type = type, intercept = intercept, x_name = c(include, x_name), y_name = y_name, weight = weight, method = test_method)
	out_init_stepwise <- getInitialStepwise(data, type = type, strategy, metric, intercept, include, x_name, y_name, weight = weight, test_method = test_method, sigma_value)
	add_or_remove <- out_init_stepwise$add_or_remove
	x_in_model <- out_init_stepwise$x_in_model
	x_notin_model <- out_init_stepwise$x_notin_model
	process_table <- out_init_stepwise$process_table
	rownames(process_table) <- NULL
	pic_df_init <- data.frame(strategy, metric, process_table[,c(1:2,6)])
	colnames(pic_df_init)[c(3:5)] <- c("step","variable","value")
	## get final stepwise model
	out_final_stepwise <- getFinalStepModel(add_or_remove, data, type = type, strategy, metric, sle, sls, weight = weight, y_name, x_in_model, x_notin_model, intercept, include, process_table, test_method, sigma_value, feature_ratio, dummy_name_list)
	
	if(type == "cox") {
		if(strategy == "backward"){
			Selection <- rep("Remove",nrow(pic_df_init))
		} else {
			pic_df_init <- pic_df_init[-1,]
			out_final_stepwise$pic_df$step <- as.numeric(out_final_stepwise$pic_df$step) - 1
			if(nrow(pic_df_init) > 0){
				Selection <- rep("Entry",nrow(pic_df_init))
				pic_df_init$step <- pic_df_init$step - 1
			} else {
				Selection <- NULL
			}
		}
	} else {
		if(strategy == "backward"){
			Selection <- rep("Remove",nrow(pic_df_init))
		} else {
			Selection <- rep("Entry",nrow(pic_df_init))
		}
	}
	for (i in out_final_stepwise$process_table$Step) {
		sub_process_table <- out_final_stepwise$process_table[out_final_stepwise$process_table$Step %in% i,]
		sub_pic_df <- out_final_stepwise$pic_df[out_final_stepwise$pic_df$step %in% i,]
		if(!is.null(sub_pic_df) && nrow(sub_pic_df) > 0){
			sub_var <- sub_process_table[,c(2,3)][!sub_process_table[,c(2,3)] %in% ""]
			if(colnames(sub_var) == "EffectEntered"){
				selected_index <- "Entry"
			} else if(colnames(sub_var) == "EffectRemoved") {
				selected_index <- "Remove"
			}
			
			sub_pic_df$value[sub_pic_df$variable %in% sub_var] <- selected_index
			sub_pic_df$value[!sub_pic_df$variable %in% sub_var] <- "No"
			Selection <- append(Selection,sub_pic_df$value)
		}
	}
	
	pic_df <- rbind(pic_df_init,out_final_stepwise$pic_df)
	pic_df$value <- as.numeric(pic_df$value)
	pic_df$step <- as.numeric(pic_df$step)
	pic_df$Selection <- Selection

	df_filtered <- do.call(rbind, lapply(split(pic_df, pic_df$step), function(g) {
	rbind(	
		# Keep max value for each unique variable in "Remove" if there are any "Remove" rows
		if (any(g$Selection == "Remove")) {
			# For each unique variable, keep the row with max value
			do.call(rbind, lapply(split(g, g$variable), function(sub_g) {
				sub_g[sub_g$value == max(sub_g$value), ]
			}))
		} else {
			g[0, ]  # empty data frame if no "Remove"
		},
		
		# Keep min value for each unique variable in "Entry" if there are any "Entry" rows
		if (any(g$Selection == "Entry")) {
			# For each unique variable, keep the row with min value
			do.call(rbind, lapply(split(g, g$variable), function(sub_g) {
				sub_g[sub_g$value == min(sub_g$value), ]
			}))
		} else {
			g[0, ]  # empty data frame if no "Entry"
		}
	)
	}))
	rownames(df_filtered) <- NULL

	out_final_stepwise$pic_df <- df_filtered
	return(out_final_stepwise)
}

getTable3ProcessSummary <- function(data_train, data_test, type, strategy, metric, sle, sls, weight, x_name, y_name, intercept, include, best_n, test_method, sigma_value, num_digits, feature_ratio, dummy_name_list) {
	overview_table_metric <- list()
	x_final_model_metric <- list()
	detail <- list()
	model_performance_df <- NULL
	overview <- list()
	
	for(stra in strategy) {
		for(met in metric) {
			if(stra == "subset") {
				overview_table <- getSubsetWrapper(data_train, type = type, met, x_name, y_name, intercept, include, weight = weight, best_n, test_method, sigma_value)
				if(met != "SL"){
					x_final_model <- getSelectedXName(overview_table,met)
				}
			} else {
				out_final_stepwise <- getStepwiseWrapper(data_train, type = type, stra, met, sle, sls, weight = weight, x_name, y_name, intercept, include, test_method, sigma_value, feature_ratio, dummy_name_list)
				detail[[stra]][[met]] <- out_final_stepwise$pic_df
				overview_table <- out_final_stepwise$process_table
				remove_col <- NULL
				if(stra == "forward") {
					remove_col <- "EffectRemoved"
				} else if(stra == "backward") {
					remove_col <- "EffectEntered"
				}
				overview_table <- overview_table[,!colnames(overview_table) %in% remove_col]
				if(all(overview_table[,"NumberEffect"] == overview_table[,"NumberParams"])) {
					overview_table <- overview_table[,!colnames(overview_table) %in% "NumberEffect"]
				}
				x_final_model <- c(intercept, include, out_final_stepwise$x_in_model)
			}
			#overview_table[,met] <- overview_table[,met] %>% as.numeric() %>% round(num_digits) %>% as.character()
			overview_table[,met] <- as.numeric(overview_table[,met])
			# Apply round to numeric columns, then convert to character
			temp_table <- overview_table
			numeric_cols <- sapply(temp_table, is.numeric)
			temp_table[numeric_cols] <- lapply(temp_table[numeric_cols], round, num_digits)
			temp_table[numeric_cols] <- lapply(temp_table[numeric_cols], as.character)
			overview[[stra]][[met]] <- temp_table # to keep digits as we expected, convert numeric to character for html output.
			
			if(!(stra == "subset" & met == "SL")) {
				if(type == 'cox') {
					x_final_model <- x_final_model[!x_final_model %in% c("1", "0", "-1")]
				}
				model_train <- getModel(data_train, type, intercept = NULL, x_name = x_final_model, y_name = y_name, weight = weight, method = test_method)
				x_final_model_metric[[stra]][[met]] <- x_final_model	
				if(type == "cox") {
				  model_performance <- cox_performance(data_train, data_test, stra, met, model_train, y_name, weight)
				} else if(type == "linear" | type == "gamma" |type == "negbin" | type == "poisson"){
				  model_performance <- lm_performance(data_train, data_test, type, stra, met, model_train, y_name, weight)
				} else if(type == "logit"){
				  model_performance <- logit_performance(data_train, data_test, stra, met, model_train, y_name, weight)
				}
			}
			model_performance_df <- rbind(model_performance_df ,model_performance)
		}
	}
	return(list('final_variable' = x_final_model_metric, 'performance' = model_performance_df, 'overview' = overview, "detail" = detail))
}

getTable4ModelCall <- function(type, intercept, include, x_final_model_metric, y_name, n_y, data, weight, test_method, num_digits) {
	table4_model_call <- list()
	for(stra in names(x_final_model_metric)) {
		x_final_model_strategy <- x_final_model_metric[[stra]]
		for(met in names(x_final_model_strategy)) {
			x_in_model <- x_final_model_strategy[[met]]
			if(type == 'cox') {
				x_in_model <- x_in_model[!x_in_model %in% c("1", "0", "-1")]
			}
			table4_model_call[[stra]][[met]] <- getModel(data, type, intercept = NULL, x_name = c(x_in_model), y_name, weight = weight,	method = test_method)
		}
	}
	return(table4_model_call)
}

cox_performance <- function(data_train, data_test, strategy, metric, model_train, y_name, weight) {	
	# 1. Concordance Index
	cindex_train <- concordance(model_train)$concordance
	
	# Test data operations with error handling
	test_results <- tryCatch({
		y_vars <- sub("Surv\\((.*)\\)", "\\1", y_name)
		time_var <- trimws(strsplit(y_vars, ",")[[1]][1])
		status_var <- trimws(strsplit(y_vars, ",")[[1]][2])
		surv_obj <- Surv(data_test[,time_var], data_test[,status_var])
		lp_test <- predict(model_train, newdata = data_test, weights=weight)
		cindex_test <- concordance(surv_obj ~ lp_test, data = data_test)$concordance
		
		# 3. Time-Dependent AUC
		data <- rbind(data_train, data_test)
		times <- seq(min(data[,time_var]), max(data[,time_var]), by = round((max(data[,time_var]) - min(data[,time_var]))/100))
		lp_train <- predict(model_train, weights=weight)
		lp_test <- predict(model_train, newdata=data_test, weights=weight)
		Surv_rsp_train <- Surv(data_train[,time_var], data_train[,status_var])
		Surv_rsp_test <- Surv(data_test[,time_var], data_test[,status_var])
		
		auc_uno <- AUC.uno(Surv_rsp_train, Surv_rsp_test, lp_test, times)
		auc_sh <- AUC.sh(Surv_rsp_train, Surv_rsp_test, lp_train, lp_test, times)
		auc_hc <- AUC.hc(Surv_rsp_train, Surv_rsp_test, lp_test, times)
		
		list(cindex_test = cindex_test, auc_hc = auc_hc$iauc, auc_uno = auc_uno$iauc, auc_sh = auc_sh$iauc)
	}, error = function(e) {
		# Return NA values if test data operations fail
		list(cindex_test = NA, auc_hc = NA, auc_uno = NA, auc_sh = NA)
	})
	
	# 2. Integrated Brier Score (requires pec package)
	# Install if needed: install.packages("pec")
	# brier_train <- pec(list(model_train), formula = Surv(time, status) ~ 1, data = data_train, 
	#                    exact = TRUE, cens.model = "cox", splitMethod = "none")
	# brier_test <- pec(list(model_train), formula = Surv(time, status) ~ 1, data = data_test, 
	#                   exact = TRUE, cens.model = "cox", splitMethod = "none")
	# cat("Integrated Brier Score (Training):", mean(brier_train$AppErr), "\n")
	# cat("Integrated Brier Score (Test):", mean(brier_test$AppErr), "\n")
	
	model_performance <- data.frame(deparse1(model_train$formula), paste0(strategy,":",metric),
								   cindex_train, test_results$cindex_test, 
								   test_results$auc_hc, test_results$auc_uno, test_results$auc_sh)
	colnames(model_performance) <- c("model", "strategy:metric", "c-index_train", "c-index_test", "auc_hc", "auc_uno", "auc_sh")
	model_performance <- model_performance[,c(1:4,7)]
	return(model_performance)
}



lm_performance <- function(data_train, data_test, type, strategy, metric, model_train, y_name, weight) {	
	pred_train <- predict(model_train, newdata=data_train, weights=weight)

	if(is.matrix(pred_train)) {
		y_name <- colnames(pred_train)
	} 
	# Actual values
	actual_train <- data_train[,y_name]
	if(type == "linear"){
	  model_train_summary <- summary(model_train)
	} else {
	  model_train_summary <- model_train
	}
	
	# Test data operations with error handling
	test_results <- tryCatch({
		pred_test <- predict(model_train, newdata = data_test, weights=weight)
		actual_test <- data_test[,y_name]
		
		if(is.matrix(pred_train)) {
		  mse_test <- colMeans((actual_test - pred_test)^2)
		  rmse_test <- sqrt(mse_test)
		  mae_test <- colMeans(abs(actual_test - pred_test))
		  r2_test <- diag(cor(actual_test, pred_test)^2)
		  n_test <- nrow(data_test)
		  coef_df <- coef(model_train)
		  p_test <- sum(!rownames(coef_df) %in% "(Intercept)")
		  adj_r2_test <- 1 - ((1 - r2_test) * (n_test - 1) / (n_test - p_test - 1))
		} else {
		  mse_test <- mean((actual_test - pred_test)^2)
		  rmse_test <- sqrt(mse_test)
		  mae_test <- mean(abs(actual_test - pred_test))
		  if(type == "linear") {
		    r2_test <- cor(actual_test, pred_test)^2
		    n_test <- nrow(data_test)
		    coef_df <- coef(summary(model_train))
		    p_test <- sum(!rownames(coef_df) %in% "(Intercept)")
		    adj_r2_test <- 1 - ((1 - r2_test) * (n_test - 1) / (n_test - p_test - 1))
		  } else {
		    r2_test <- NA
			adj_r2_test <- NA
		  }
		}
		list(pred_test = pred_test, actual_test = actual_test, mse_test = mse_test, 
			 rmse_test = rmse_test, mae_test = mae_test, r2_test = r2_test, adj_r2_test = adj_r2_test)
	}, error = function(e) {
		# Return NA values if test data operations fail
		list(pred_test = NA, actual_test = NA, mse_test = NA, 
			 rmse_test = NA, mae_test = NA, r2_test = NA, adj_r2_test = NA)
	})
  
	if(is.matrix(pred_train)) {
	  mse_train <- colMeans((model_train$residuals)^2)
	  rmse_train <- sqrt(mse_train)
	  mae_train <- colMeans(abs(model_train$residuals))
	  #r2_train <- model_train_summary$adj.r.squared # cannot access adj.r.squared of two response
	  adj_r2_train <- NULL
	  for(i in 1:dim(pred_train)[2]) {
	    adj_r2_train <- append(adj_r2_train, model_train_summary[[i]]$adj.r.squared)
	  }
	  names(adj_r2_train) <- y_name
	} else {
	  mse_train <- mean((model_train_summary$residuals)^2)
	  rmse_train <- sqrt(mse_train)
	  mae_train <- mean(abs(model_train_summary$residuals))
	  if(type == "linear") {
	    adj_r2_train <- model_train_summary$adj.r.squared
	  } else {
	    adj_r2_train <- NA
	  }
	}

	model_performance <- data.frame(deparse1(model_train$call$formula), paste0(strategy,":",metric), 
								   adj_r2_train, test_results$adj_r2_test, 
								   mse_train, test_results$mse_test, 
								   mae_train, test_results$mae_test
								   )
	if(is.matrix(pred_train)) {
	  model_performance$response <- y_name
	  colnames(model_performance) <- c("model", "strategy:metric", "adjR-squared_train", "adjR-squared_test", "mse_train", "mse_test", "mae_train", "mae_test", "response")
	} else {
	  colnames(model_performance) <- c("model", "strategy:metric", "adjR-squared_train", "adjR-squared_test", "mse_train", "mse_test", "mae_train", "mae_test")
	}
	if(type != "linear") {
	  model_performance <- model_performance[,-c(3:4)]
	}
	return(model_performance)
}



logit_performance <- function(data_train, data_test, strategy, metric, model_train, y_name, weight) {	
	# Predictions
	pred_prob_train <- predict(model_train, newdata=data_train, type = "response", weights=weight)
	
	# Test data operations with error handling
	test_results <- tryCatch({
		pred_prob_test <- predict(model_train, newdata = data_test, type = "response", weights=weight)
		pred_class_test <- ifelse(pred_prob_test > 0.5, 0, 1)
		actual_test <- data_test[,y_name]
		conf_mat_test <- table(Predicted = pred_class_test, Actual = actual_test)
		accuracy_test <- sum(diag(conf_mat_test)) / sum(conf_mat_test)
		auc_test <- tryCatch({
			auc(roc(actual_test, pred_prob_test))
		}, error = function(e) {
			NA  # Return NA if AUC calculation fails
		})
		log_loss_test <- -mean(actual_test * log(pred_prob_test + 1e-15) + (1 - actual_test) * log(1 - pred_prob_test + 1e-15))
		
		list(pred_prob_test = pred_prob_test, pred_class_test = pred_class_test, 
			 actual_test = actual_test, conf_mat_test = conf_mat_test, 
			 accuracy_test = accuracy_test, auc_test = auc_test, log_loss_test = log_loss_test)
	}, error = function(e) {
		# Return NA values if test data operations fail
		list(pred_prob_test = NA, pred_class_test = NA, 
			 actual_test = NA, conf_mat_test = NA, 
			 accuracy_test = NA, auc_test = NA, log_loss_test = NA)
	})
	
	pred_class_train <- ifelse(pred_prob_train > 0.5, 1, 0)

	# Actual values
	actual_train <- data_train[,y_name]

	# Metrics
	conf_mat_train <- table(Predicted = pred_class_train, Actual = actual_train)
	accuracy_train <- sum(diag(conf_mat_train)) / sum(conf_mat_train)
	
	# Calculate AUC with error handling
	auc_train <- tryCatch({
		auc(roc(actual_train, pred_prob_train))
	}, error = function(e) {
		NA  # Return NA if AUC calculation fails
	})
	
	log_loss_train <- -mean(actual_train * log(pred_prob_train + 1e-15) + (1 - actual_train) * log(1 - pred_prob_train + 1e-15))

	model_performance <- data.frame(deparse1(model_train$formula), paste0(strategy,":",metric), 
								   accuracy_train, test_results$accuracy_test, 
								   auc_train, test_results$auc_test, 
								   log_loss_train, test_results$log_loss_test)
	colnames(model_performance) <- c("model", "strategy:metric", "accuracy_train", "accuracy_test", "auc_train", "auc_test", "log_loss_train", "log_loss_test")
	return(model_performance)
}

#' Generate Stepwise Regression Report
#'
#' Creates formatted reports from StepReg objects in various document formats. This function
#' generates comprehensive reports containing all tables and results from the stepwise regression
#' analysis.
#'
#' @param x A StepReg object containing the results of stepwise regression analysis.
#' 
#' @param report_name Character. The name of the output report file(s) without extension.
#' 
#' @param format Character vector. The output format(s) for the report. Choose from:
#'   \itemize{
#'     \item "html" - Web page format (default)
#'     \item "docx" - Microsoft Word document
#'     \item "pptx" - Microsoft PowerPoint presentation
#'     \item "rtf" - Rich Text Format
#'   }
#'   Multiple formats can be specified simultaneously.
#'
#' @details The generated report includes:
#'   \itemize{
#'     \item Summary of model parameters and selection criteria
#'     \item Variable types and classifications
#'     \item Step-by-step selection process
#'     \item Final selected model and fit statistics
#'     \item Model coefficients and significance levels
#'   }
#'   
#'   \strong{Note:} Exporting reports requires pandoc to be installed on your system. 
#'   Pandoc is typically included with RStudio. If not available, install it from 
#'   https://pandoc.org/installing.html or via your system package manager.
#'
#' @return Creates report file(s) in the specified format(s) in the current working directory.
#' The file name will be \code{report_name.format} (e.g., "myreport.html", "myreport.docx").
#'
#' @importFrom flextable save_as_html save_as_pptx save_as_rtf save_as_docx autofit flextable align
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' # Load leukemia remission data
#' data(remission)
#' 
#' # Run stepwise logistic regression
#' formula <- remiss ~ .
#' result <- stepwise(
#'   formula = formula,
#'   data = remission,
#'   type = "logit",
#'   strategy = c("forward", "bidirection"),
#'   metric = c("AIC", "BIC")
#' )
#' 
#' # Generate reports in multiple formats
#' report(
#'   x = result,
#'   report_name = "leukemia_analysis",
#'   format = c("html", "docx")
#' )
#' }
#'
#' @seealso \code{\link{stepwise}} for creating StepReg objects
#' @seealso \code{\link{plot.StepReg}} for visualization of results

report <- function(x, report_name, format = c('html', 'docx', 'rtf', 'pptx')) {
  format <- match.arg(format, several.ok = TRUE)
  
  # Check if pandoc is available (required for flextable export functions)
  if (any(c('html', 'docx', 'rtf', 'pptx') %in% format)) {
    if (!requireNamespace("rmarkdown", quietly = TRUE)) {
      stop("The 'rmarkdown' package is required to check pandoc availability. Please install it with: install.packages('rmarkdown')")
    }
    if (!rmarkdown::pandoc_available()) {
      stop("Pandoc is not available. The flextable export functions require pandoc to be installed.\n",
           "Please install pandoc using one of the following methods:\n",
           "  1. Install RStudio (which includes pandoc): https://www.rstudio.com/products/rstudio/download/\n",
           "  2. Install pandoc directly from: https://pandoc.org/installing.html\n",
           "  3. On macOS with Homebrew: brew install pandoc\n",
           "  4. On Linux: Use your system package manager (e.g., sudo apt-get install pandoc)")
    }
  }
  
  results <- list()
  for (j in c("argument","variable")) {
    if(j == "argument") {
      j_name <- "Parameters and Values"
    } else if(j == "variable"){
      j_name <- "variable and Class"
    }
    results[j_name] <- list(process_table(x[[j]]))
  }
  
  overview_list <- x$overview
  for(i in names(overview_list)){
    overview_sub_list <- overview_list[[i]]
    for(j in names(overview_sub_list)){
      results[paste("Selection Overview: ",paste(i,j,sep="-"))] <- list(process_table(overview_sub_list[[j]]))
    }
  }
  results["Model Performance"] <- list(process_table(performance(x)))
  names(results) <- paste("Table", paste(1:length(results),names(results)),sep="")
  
  if(!is.null(report_name)) {
    if (any(c('html', 'docx', 'rtf', 'pptx') %in% format)) {
      for (i in format) {
        if (i %in% 'html') {
          save_as_html(values = results,
                       path = paste0(report_name, ".html"))
        } 
        if (i %in% 'docx') {
          save_as_docx(values = results,
                       path = paste0(report_name, ".docx"))
        } 
        if (i %in% 'rtf') {
          save_as_rtf(values=results,
                      path = paste0(report_name, ".rtf"))
        } 
        if (i %in% 'pptx') {
          save_as_pptx(values = results,
                       path = paste0(report_name, ".pptx"))
        }
        # if (i %in% 'xlsx') {
        #   wb <- createWorkbook()
        #   cs <- CellStyle(wb) + Font(wb, isBold = TRUE)
        #   #k=2
        #   for (k in 1:length(x)) {
        #     x[[k]] <- as.data.frame(x[[k]]) # tbl won't work properly when addDataFrame
        #     x[[k]] <- x[[k]] %>% mutate_if(is.numeric, as.character) # Inf will not display properly in excel, so convert numeric to character
        #     
        #     sheet <- createSheet(wb, paste0("table", k))
        #     # Add sheet title and content:
        #     addDataFrame(data.frame(Dataframe = names(x)[[k]]), sheet, startRow = 1, startColumn = 1, col.names = FALSE, row.names = FALSE, colnamesStyle = cs)
        #     addDataFrame(x[[k]], sheet, startRow = 2, row.names = FALSE, colnamesStyle = cs)
        #   }
        #   saveWorkbook(wb, file = paste0(report_name, ".xlsx"))
        # }
      }
    }
  }
}

process_table <- function(data) {
  align(autofit(flextable(as.data.frame(data))), align = "center", part = "all")
}

#' Fair's Extramarital Affairs Data
#'
#' Infidelity data, known as Fair's Affairs. Cross-section data from a survey conducted by Psychology Today in 1969. This dataset is sourced from the AER package.
#'
#' @format A data frame containing 601 observations on 9 variables.
#' \describe{
#'   \item{affairs}{numeric. How often engaged in extramarital sexual intercourse during the past year? 0 = none, 1 = once, 2 = twice, 3 = 3 times, 7 = 4–10 times, 12 = monthly, 12 = weekly, 12 = daily.}
#'   \item{gender}{factor indicating gender.}
#'   \item{age}{numeric variable coding age in years: 17.5 = under 20, 22 = 20–24, 27 = 25–29, 32 = 30–34, 37 = 35–39, 42 = 40–44, 47 = 45–49, 52 = 50–54, 57 = 55 or over.}
#'   \item{yearsmarried}{numeric variable coding number of years married: 0.125 = 3 months or less, 0.417 = 4–6 months, 0.75 = 6 months–1 year, 1.5 = 1–2 years, 4 = 3–5 years, 7 = 6–8 years, 10 = 9–11 years, 15 = 12 or more years.}
#'   \item{children}{factor. Are there children in the marriage?}
#'   \item{religiousness}{numeric variable coding religiousness: 1 = anti, 2 = not at all, 3 = slightly, 4 = somewhat, 5 = very.}
#'   \item{education}{numeric variable coding level of education: 9 = grade school, 12 = high school graduate, 14 = some college, 16 = college graduate, 17 = some graduate work, 18 = master's degree, 20 = Ph.D., M.D., or other advanced degree.}
#'   \item{occupation}{numeric variable coding occupation according to Hollingshead classification (reverse numbering).}
#'   \item{rating}{numeric variable coding self rating of marriage: 1 = very unhappy, 2 = somewhat unhappy, 3 = average, 4 = happier than average, 5 = very happy.}
#' }
#'
#' @source Fair, R.C. (1978). A Theory of Extramarital Affairs. Journal of Political Economy, 86, 45–61.
#'
#' @seealso \code{\link[AER]{Affairs}} for the original dataset in the AER package
#' 
#' @references 
#' 
#' \itemize{
#'   \item Greene, W.H. (2003). Econometric Analysis, 5th edition. Upper Saddle River, NJ: Prentice Hall.
#' 
#' \item Fair, R.C. (1978). A Theory of Extramarital Affairs. Journal of Political Economy, 86, 45–61.
#' }
#' 
#' @examples
#' data(affairs)
#' summary(affairs)
#' @usage data(affairs)
#' @name affairs
#' @docType data
#' @keywords datasets
"affairs"

#' Visualize Stepwise Regression Results
#'
#' Creates informative visualizations of the variable selection process from a StepReg object.
#' This function generates two types of plots: detailed step-by-step selection process and
#' an overview of the final selected variables.
#'
#' @param x A StepReg object containing the results of stepwise regression analysis.
#' 
#' @param strategy Character. Specifies which selection strategy to visualize:
#'   \itemize{
#'     \item "forward" - Forward selection
#'     \item "backward" - Backward elimination
#'     \item "bidirection" - Bidirectional selection
#'     \item "subset" - Best subset selection
#'   }
#'   Default is the first strategy name in the StepReg object.
#' 
#' @param process Character. Specifies the type of visualization to display:
#'   \itemize{
#'     \item "detail" - Shows detailed step-by-step selection process with variable entry/removal
#'     \item "overview" - Shows summary of the selection process with metric values
#'   }
#'   Default is "overview".
#' 
#' @param num_digits Integer. Number of decimal places to display in the plots.
#'   Default is 6.
#' 
#' @param ... Additional argument passed to plotting functions (currently not used).
#'
#' @return A ggplot object showing either:
#'   \itemize{
#'     \item For "detail" process: A heatmap showing variable selection status at each step
#'     \item For "overview" process: A line plot showing metric values across steps
#'   }
#' 
#' @details The function creates different types of visualizations based on the selection strategy:
#'   \itemize{
#'     \item For forward/backward/bidirectional selection:
#'       \itemize{
#'         \item detail view shows a heatmap with green tiles for added variables,
#'               tan tiles for removed variables, and gray tiles for non-selected variables
#'         \item Overview shows metric values across steps with variable labels
#'       }
#'     \item For subset selection:
#'       \itemize{
#'         \item detail view shows a heatmap of selected variables at each step
#'         \item Overview shows metric values for different subset sizes
#'       }
#'   }
#' 
#' @import ggplot2
#' @importFrom ggrepel geom_label_repel
#' 
#' @export
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data(mtcars)
#' 
#' # Run stepwise regression with multiple strategies
#' formula <- mpg ~ .
#' result <- stepwise(
#'   formula = formula,
#'   data = mtcars,
#'   type = "linear",
#'   strategy = c("forward", "bidirection", "subset"),
#'   metric = c("AIC", "BIC", "SL")
#' )
#' 
#' # Generate default overview plot
#' plot(result)
#' 
#' # Generate detailed plot for forward selection
#' plot(result, strategy = "forward", process = "detail")
#' 
#' # Generate overview plot with 3 decimal places
#' plot(result, strategy = "bidirection", process = "overview", num_digits = 3)
#' }
#'
#' @seealso \code{\link{stepwise}} for creating StepReg objects

plot.StepReg <- function(x, strategy = attr(x,"nonhidden"), process = c("overview", "detail"), num_digits = 6, ...) {
  process <- match.arg(process)
  strategy <- match.arg(strategy)
  
  detail_list <- x$detail
  argument <- x$argument
  test_method <- argument[argument$Parameter %in% "test method",2]
  overview_list <- x$overview
  strategy_vec <- unlist(strsplit(argument[argument$Parameter %in% "selection strategy",2]," & "))
  
  if(!strategy %in% strategy_vec){
    stop(paste0(strategy," is not found in StepReg object"))
  }
  
  overview_process_table <- overview_list[[strategy]]
  if (strategy == "subset") {
    overview_df <- lapply(overview_process_table, function(df) {
      df <- cbind(df,colnames(df)[2])
      colnames(df) <- c("Step", "MetricValue", "Variable", "Metric")
      return(df)
    })
  } else {
    overview_df <- lapply(overview_process_table, function(df) {
      df <- cbind(df,colnames(df)[ncol(df)])
      var_sym <- rep(NA, nrow(df))
      var_plus <- paste0("(+)", df[, colnames(df) %in% "EffectEntered"])
      index_plus <- which(!var_plus %in% "(+)")
      var_minus <- paste0("(-)", df[, colnames(df) %in% "EffectRemoved"])
      index_minus <- which(!var_minus %in% "(-)")
      var_sym[index_plus] <- var_plus[index_plus]
      var_sym[index_minus] <- var_minus[index_minus]
      df <- cbind(df[,c(1,ncol(df)-1,ncol(df))],var_sym)
      colnames(df) <- c("Step", "MetricValue", "Metric", "Variable")
      return(df)
    })
  }
  names(overview_df) <- NULL
  plot_overview <- do.call(rbind, overview_df)
  plot_overview$Step <- as.numeric(plot_overview$Step)
  plot_overview$MetricValue <- as.numeric(plot_overview$MetricValue)
  
  if(strategy %in% names(detail_list)) {
    detail_process_table <- detail_list[[strategy]]
    plot_detail <- do.call(rbind, detail_process_table)
  }
  
  if(process == "detail"){
    if(strategy != "subset") {
      p1 <- plotStepwiseDetail(plot_detail, num_digits)
    } else {
      #From SAS description: For two models A and B, each having the same number of explanatory variables, model A is considered to be better than model B if the global score chi-square statistic for A exceeds that for B.
      # Group by Step and Metric, then filter based on condition
      # For SL metric, keep rows with max MetricValue; for others, keep rows with min MetricValue
      plot_overview <- plot_overview[order(plot_overview$Step, plot_overview$Metric, 
                                          ifelse(plot_overview$Metric == "SL", -plot_overview$MetricValue, plot_overview$MetricValue)), ]
      plot_overview <- plot_overview[!duplicated(plot_overview[, c("Step", "Metric")]), ]
      p1 <- plotSubsetDetail(plot_overview)
    }
    return(p1)
  } else {
    ## make a dual y-axis with log10 transformed for 'SL' if 'SL' is selected
    if(strategy != 'subset') {
      if("SL" %in% plot_overview$Metric) {
        plot_overview[plot_overview$Metric == "SL",]$MetricValue <- plot_detail[plot_detail$metric == "SL" & plot_detail$Selection %in% c("Entry","Remove"),"value"]
        # Handle Inf values only if there are non-Inf values
        inf_indices <- which(is.infinite(plot_overview$MetricValue))
        if(length(inf_indices) > 0) {
          non_inf_values <- plot_overview$MetricValue[!is.infinite(plot_overview$MetricValue) & !is.na(plot_overview$MetricValue)]
          if(length(non_inf_values) > 0) {
            plot_overview$MetricValue[inf_indices] <- max(non_inf_values) * 1.1
          } else {
            plot_overview$MetricValue[inf_indices] <- 1.0  # Default value if all are Inf
          }
        }
        
        # Calculate range for SL metric, filtering out NA and Inf values
        sl_values <- plot_overview[plot_overview$Metric == "SL", ]$MetricValue
        sl_values <- sl_values[!is.na(sl_values) & !is.infinite(sl_values) & sl_values > 0]
        if(length(sl_values) > 0) {
          a <- range(log10(sl_values))
        } else {
          a <- c(0, 1)  # Default range if no valid SL values
        }
        
        # Calculate range for non-SL metrics, filtering out NA and Inf values
        non_sl_values <- plot_overview[plot_overview$Metric != "SL", ]$MetricValue
        non_sl_values <- non_sl_values[!is.na(non_sl_values) & !is.infinite(non_sl_values)]
        if(length(non_sl_values) > 0) {
          b <- range(non_sl_values)
        } else {
          b <- c(0, 1)  # Default range if no valid non-SL values
        }
        
        p2 <- plotStepwiseSummaryDualY(x, plot_overview, a, b, strategy)
      } else {
        p2 <- plotStepwiseSummarySingleY(plot_overview)
      }
    } else {
      p2 <- plotSubsetSummary(plot_overview, test_method)
    }
    p2 <- p2 +
      scale_x_continuous(breaks = plot_overview$Step) + 
      labs(title =paste0("Selection overview: ",strategy)) + 
      theme_light()
    return(p2)
  }
}

plotStepwiseSummaryDualY <- function(x, df, a, b, n){
  # Filter out rows with empty or NA Variable values for label repelling
  df_labels <- df[!is.na(df$Variable) & df$Variable != "", ]
  
  # Ensure diff(a) and diff(b) are not zero to avoid division by zero
  if(diff(a) == 0) {
    a[2] <- a[1] + 1
  }
  if(diff(b) == 0) {
    b[2] <- b[1] + 1
  }
  
  if(all(df$Metric %in% "SL")) {
    # Filter valid SL values for breaks calculation
    valid_sl_values <- df$MetricValue[!is.na(df$MetricValue) & !is.infinite(df$MetricValue) & df$MetricValue > 0]
    
    p2 <- ggplot(df, aes(x = .data$Step, group = .data$Metric)) +
      geom_point(aes(y = (log10(.data$MetricValue) - a[1])/diff(a), color = .data$Metric)) + 
      geom_label_repel(data = df_labels,
                       aes(y = (log10(.data$MetricValue) - a[1])/diff(a), color = .data$Metric, label = .data$Variable),
                       label.size = 0.05,
                       show.legend = FALSE) + 
      scale_y_continuous(
        labels = function(x) sprintf("%.1e", 10^(x * diff(a) + a[1])),
        breaks = if(length(valid_sl_values) > 0) (pretty(log10(valid_sl_values)) - a[1])/diff(a) else NULL, 
        name = "SL (p value)") + 
      geom_line(aes(y = (log10(.data$MetricValue) - a[1])/diff(a), color = .data$Metric))
  } else {
    # Filter valid values for breaks calculation
    valid_sl_values <- df[df$Metric == "SL", ]$MetricValue
    valid_sl_values <- valid_sl_values[!is.na(valid_sl_values) & !is.infinite(valid_sl_values) & valid_sl_values > 0]
    
    p2 <- ggplot(df, aes(x = .data$Step, group = .data$Metric)) +
      geom_point(aes(y = ifelse(.data$Metric == "SL", (log10(.data$MetricValue) - a[1])/diff(a), (.data$MetricValue - b[1])/diff(b)), color = .data$Metric)) + 
      geom_label_repel(data = df_labels,
                       aes(y = ifelse(.data$Metric == "SL", (log10(.data$MetricValue) - a[1])/diff(a), (.data$MetricValue - b[1])/diff(b)), color = .data$Metric, label = .data$Variable),
                       label.size = 0.05,
                       show.legend = FALSE) + 
      scale_y_continuous(
        labels = function(x) sprintf("%.1e", 10^(x * diff(a) + a[1])),
        breaks = if(length(valid_sl_values) > 0) (pretty(log10(valid_sl_values)) - a[1])/diff(a) else NULL, 
        name = "SL (p value)",
        sec.axis = sec_axis(~(diff(b) * . + b[1]), name = paste0(unique(df$Metric)[!unique(df$Metric) == "SL"],collapse=" / "))) + 
      geom_line(aes(y = ifelse(.data$Metric == "SL", (log10(.data$MetricValue) - a[1])/diff(a), (.data$MetricValue - b[1])/diff(b)), color = .data$Metric))
  }
  if(n == "forward") {
    sle <- x[[1]][which(x[[1]][,1] %in% "significance level for entry (sle)"), 2]
    p2 <- p2 + 
      geom_hline(yintercept=(log10(as.numeric(sle)) - a[1])/diff(a), linetype="dashed", color = "gray") + 
      geom_text(aes(1.5,(log10(as.numeric(sle)) - a[1])/diff(a), label = paste0("sle=",sle), vjust = -1), color = "gray")
  } else if (n == "backward") {
    sls <- x[[1]][which(x[[1]][,1] %in% "significance level for stay (sls)"), 2]
    p2 <- p2 + 
      geom_hline(yintercept=(log10(as.numeric(sls)) - a[1])/diff(a), linetype="dotdash", color = "gray") + 
      geom_text(aes(max(as.numeric(.data$Step)) - 0.5, (log10(as.numeric(sls)) - a[1])/diff(a), label = paste0("sls=",sls), vjust = -1), color = "gray")
  } else {
    sle <- x[[1]][which(x[[1]][,1] %in% "significance level for entry (sle)"), 2]
    sls <- x[[1]][which(x[[1]][,1] %in% "significance level for stay (sls)"), 2]
    p2 <- p2 +
      geom_hline(yintercept=(log10(as.numeric(sle)) - a[1])/diff(a), linetype="dashed", color = "gray") + 
      geom_text(aes(1.5,(log10(as.numeric(sle)) - a[1])/diff(a), label = paste0("sle=",sle), vjust = -1), color = "gray") + 
      geom_hline(yintercept=(log10(as.numeric(sls)) - a[1])/diff(a), linetype="dotdash", color = "gray") + 
      geom_text(aes(max(as.numeric(.data$Step)) - 0.5, (log10(as.numeric(sls)) - a[1])/diff(a), label = paste0("sls=",sls), vjust = -1), color = "gray")
  }
  p2 <- p2 + xlab("Step")
  return(p2)
}

plotStepwiseSummarySingleY <- function(df){
  # Filter out rows with empty or NA Variable values for label repelling
  df_labels <- df[!is.na(df$Variable) & df$Variable != "", ]
  
  metricValue <- unique(df$Metric)
  if("SL" %in% metricValue){
    metricValue[which(metricValue %in% "SL")] <- "SL (p value)"
  }
  p2 <- ggplot(data = df) + 
    aes(x = .data$Step,
        y = .data$MetricValue, 
        label = .data$Variable,
        group = .data$Metric) + 
    geom_point(aes(color = .data$Metric)) + 
    geom_line(aes(linetype = .data$Metric, color = .data$Metric)) +
    geom_label_repel(data = df_labels,
                     label.size = 0.05,
                     aes(color = .data$Metric),
                     show.legend = FALSE) + 
    xlab("Step") +
    ylab(paste0(metricValue,collapse=" / "))
}

plotSubsetSummary <- function(df, test_method) {
  metricValue <- unique(df$Metric)
  if("SL" %in% metricValue){
    metricValue[metricValue %in% "SL"] <- paste0("SL (",test_method," statistics)")
  }
  p2 <- ggplot(data = df) + 
    aes(x = .data$Step,
        y = .data$MetricValue, 
        label = .data$Variable,
        group = .data$Metric) + 
    geom_point(aes(color = .data$Metric)) + 
    geom_line(aes(linetype = .data$Metric, color = .data$Metric)) +
    xlab("Variable number") +
    ylab(paste0(metricValue,collapse=" / "))
  return(p2)
}

plotStepwiseDetail <- function(df, num_digits) {
  p1 <- ggplot(df, 
               aes(x = .data$step,
                   y = .data$variable)) +
    geom_tile(aes(fill = .data$Selection), width = 0.99, height = 0.95, color = "black") +
    geom_text(aes(label = round(.data$value, num_digits)),
              color = "black",
              size = 2) +
    scale_fill_manual(values = c("Entry" = "palegreen2", "Remove" = "tan3","No" = "gray80")) +
    theme_light() + 
    scale_x_continuous(breaks = unique(df$step)) + 
    theme(axis.text.y = element_text(size = 8),
          strip.text = element_text(color = "black")) +  # Adjust text color in facet labels
    facet_wrap(~ .data$metric, ncol=1) + 
    theme(strip.background = element_rect(colour = "black", fill = "gray80")) +
    ggtitle(paste0("Selection details: ", df$strategy[1])) + 
    ylab("Predictors") + 
    xlab("Step")
  return(p1)
}

plotSubsetDetail <- function(plot_overview) {
  #-------------------------
  #subset works for SL in logit(need to test)
  variable_list <- lapply(strsplit(plot_overview$Variable, " "), function(x) x[x != ""])
  tile_df <- expand.grid(Variable = variable_list[[length(variable_list)]], Step = plot_overview$Step)
  tile_df$Metric <- rep(plot_overview$Metric, each = length(variable_list[[length(variable_list)]]))
  tile_df$Selection <- mapply(function(metric, step, variable) {
    df2_1 <- plot_overview[plot_overview$Metric == metric, ]
    df2_2 <- df2_1[df2_1$Step == step, ]
    any(variable %in% strsplit(df2_2$Variable, " ")[[1]])
  }, tile_df$Metric, tile_df$Step, tile_df$Variable)
  tile_df$Selection <- ifelse(tile_df$Selection, "Entry", "No")
  
  p1 <- ggplot(tile_df, aes(x = .data$Step, y = .data$Variable, fill = .data$Selection)) +
    geom_tile(width = 0.99, height = 0.95, color = "black") +
    scale_fill_manual(values = c("Entry" = "palegreen2", "No" = "gray80")) +
    labs(x = "Step", y = "Predictors", title = "Selection details: subset") +
    scale_x_continuous(breaks = plot_overview$Step) + 
    xlab("Variable number") +
    facet_wrap(~ .data$Metric, ncol=1) + 
    theme(strip.background = element_rect(colour = "black", fill = "gray80"))
}

#' Tobacco Leaf Chemical Composition Dataset
#'
#' A dataset containing chemical composition measurements from 25 tobacco leaf samples.
#' This dataset is commonly used for demonstrating multivariate regression analysis
#' and exploring relationships between chemical components and burn rate.
#'
#' @format A data frame with 25 rows and 9 variables:
#' \describe{
#'   \item{cigarette}{Numeric. Rate of cigarette burn in inches per 1000 seconds.}
#'   \item{sugar}{Numeric. Percentage of sugar content in the leaf.}
#'   \item{nicotine}{Numeric. Percentage of nicotine content.}
#'   \item{nitrogen}{Numeric. Percentage of nitrogen content.}
#'   \item{chlorine}{Numeric. Percentage of chlorine content.}
#'   \item{potassium}{Numeric. Percentage of potassium content.}
#'   \item{phosphorus}{Numeric. Percentage of phosphorus content.}
#'   \item{calcium}{Numeric. Percentage of calcium content.}
#'   \item{magnesium}{Numeric. Percentage of magnesium content.}
#' }
#'
#' @details This dataset is particularly useful for:
#' \itemize{
#'   \item Demonstrating multivariate regression analysis
#'   \item Studying relationships between chemical composition and burn rate
#'   \item Analyzing correlations between different chemical components
#'   \item Teaching statistical methods in agricultural research
#' }
#'
#' @source Anderson, R. L. and Bancroft, T. A. (1952), Statistical Theory in Research,
#' McGraw-Hill Book Company, Inc., New York, NY.
#'
#' @examples
#' # Load the dataset
#' data(tobacco)
#' 
#' # View the first few rows
#' head(tobacco)
#' 
#' # Summary statistics
#' summary(tobacco)
#' 
#' # Correlation analysis
#' cor(tobacco)
#'
#' @keywords datasets
#' @docType data
#' @usage data(tobacco)
"tobacco"

# Input validation
# Validate the input parameters passed to stepwise()
# Stop and exit if any error detected in input parameters
# @author Kai Hu, Junhui Li
validateUtils <- function(formula,
                          data,
                          type = c("linear", "logit", "poisson", "cox", "gamma", "negbin"),
                          include = NULL,
                          strategy = c("forward", "backward", "bidirectional", "subset"),
                          metric = c("AIC", "AICc", "BIC", "CP", "HQ", "HQc", "adjRsq", "SL", "SBC", "IC(3/2)", "IC(1)"),
                          sle = 0.15,
                          sls = 0.15,
                          sigma_value,
                          test_method_linear = c("Pillai", "Wilks", "Hotelling-Lawley", "Roy"),
                          test_method_glm = c("Rao", "LRT"),
                          test_method_cox = c("efron", "breslow", "exact"),
                          tolerance = 10e-7,
                          weight = NULL,
                          best_n = Inf,
                          n_y,
                          num_digits = 6) {
	## check required parameters
	if(missing(data)) { 
		stop("'data' parameter is missing.") 
	}else{
		if(!inherits(data, "data.frame")) {
			stop("'data' should be a data.frame class.")
		}
	}
	
	if(missing(formula)) { 
		stop("'formula' parameter is missing.") 
	}else{
		if(!inherits(formula, "formula")) {
			stop("'formula' should be a formula class.")
		}
	}
	# response_variable_n <<- length(all.vars(formula[[2]]))
	
	 # Ref: https://stackoverflow.com/questions/13217322/how-to-reliably-get-dependent-variable-name-from-formula-object
	
	if(!is.null(include)) {
		term_form <- terms(formula, data = data)
		x_name <- attr(term_form, "term.labels")
		y_name <- getYname(formula, data)
		formula_include <- reformulate(include, response = y_name)
		term_form_include <- terms(formula_include, data = data)
		x_name_include <- attr(term_form_include, "term.labels")
		
		# Normalize interaction terms to alphabetical order for comparison
		normalize_interaction_terms <- function(terms) {
			sapply(terms, function(term) {
				if (grepl(":", term)) {
					parts <- strsplit(term, ":")[[1]]
					paste(sort(parts), collapse = ":")
				} else if(grepl("\\*", term)) {
					parts <- strsplit(term, "\\*")[[1]]
					paste(sort(parts), collapse = "*")
				} else {
					term
				}
			})
		}
		
		x_name_normalized <- normalize_interaction_terms(x_name)
		x_name_include_normalized <- normalize_interaction_terms(x_name_include)
		
		## check if the include variables are in the formula
		if(!all(x_name_include_normalized %in% x_name_normalized)) {
			stop(paste0("'include' should be a subset of: c('",paste0(x_name,collapse = "','"),"')"))
		}
	}
	
	if(is.numeric(sle) & is.numeric(sls)) {
		if(sle <= 0 | sle > 1) {
			stop("'sle' shoule be a value from 0 to 1.")
		}
		if(sls <= 0 | sls > 1) {
			stop("'sls' shoule be a value from 0 to 1.")
		}
	}else{
		stop("'sle' and 'sls' should be numeric.")
	}
	
	## check 'metric' and 'test_method' according to 'type'
	linear_metric <- c("AIC", "AICc", "BIC", "CP", "HQ", "HQc", "adjRsq", "SL", "SBC", "IC(3/2)", "IC(1)")
	glm_metric <- c("SL", "AIC", "AICc", "SBC", "HQ", "HQc", "IC(3/2)", "IC(1)")
	#poisson_metric <- c("SL", "AIC", "AICc", "SBC", "HQ", "HQc", "IC(3/2)", "IC(1)")
	#gamma_metric <- c("SL", "AIC", "AICc", "SBC", "HQ", "HQc", "IC(3/2)", "IC(1)")
	cox_metric <- c("SL", "AIC", "AICc", "SBC", "HQ", "HQc", "IC(3/2)", "IC(1)")
	
	if(type == "linear") {
		if(any(!metric %in% linear_metric)) {
			stop("for type 'linear': 'metric' should come from the c('", paste0(linear_metric, collapse = "','"),"').")
		}
	  if(any(metric %in% c("BIC","CP"))){
	    x_name_orig <- getXname(formula, data)
	    y_name <- getYname(formula, data)
	    intercept <- getIntercept(formula, data, type = type) # char type
	    merged_include <- getMergedVar(include)
	    model_raw <- getModel(data, type = type, intercept = intercept, x_name_orig, y_name, weight = weight, method = test_method_cox)
	    if(model_raw$rank >= nrow(data)) {
	      stop("The 'metric' can not be 'CP' or 'BIC' when variable number is greater than the number of observation.")
	    }
	  }
	  if(n_y > 1) {
	    if(any(metric %in% c("BIC", "CP", "HQc", "IC(1)", "IC(3/2)", "adjRsq"))) {
	      stop("The 'metric' can not be 'BIC', 'CP', 'HQc', 'IC(1)', 'IC(3/2)', or 'adjRsq' when using multivariate multiple regression!")
	    }
	  }
		# if(any(strategy == "subset") & any(metric == "SL")) {
		# 	stop("metric = 'SL' is not allowed when strategy = 'subset'")
		# }
	  if(any(metric == "CP") & sigma_value == 0) {
	    stop("metric = 'CP' is not allowed when Estimate of pure error variance from fitting the full model(sigma_value) is 0")
	  }
	}else if(type == "logit" | type == "poisson" | type == "gamma") {
		if(any(!metric %in% glm_metric)) {
			stop("for type ",type,": 'metric' should come from the c('", paste0(glm_metric, collapse = "','"),"').")
		}
	  type_glm <- switch(type,
	                     "logit" = "binomial",
	                     "poisson" = "poisson",
	                     "gamma" = "Gamma"
	  )
	                     
	  # check if Y separates X completely, if so, stop
	  # ref: https://stats.oarc.ucla.edu/other/mult-pkg/faq/general/faqwhat-is-complete-or-quasi-complete-separation-in-logistic-regression-and-what-are-some-strategies-to-deal-with-the-issue/
	  tryCatch(                
	    expr = {                      
	      if (is.null(weight)) {
	        glm(formula, data = data, weights = NULL, family = type_glm)
	      } else {
	        glm(formula, data = data, weights = weight, family = type_glm)
	      }
	    },
	    error = function(e) {          
	      print("There was an error message.")
	    },
	    warning = function(w) {  
	      if (w$message %in% c("glm.fit: fitted probabilities numerically 0 or 1 occurred", "glm.fit: algorithm did not converge")) {
	        stop("There is a perfect separation of data points, can not obtain a valid model fit.")
	      }
	    }
	  )
	}else if(type == 'cox') {
		if(any(!metric %in% cox_metric)) {
			stop("for type 'cox': 'metric' should come from the c('", paste0(cox_metric, collapse = "','"),"').")
		}
	}else if(type == 'negbin') {
	  if(any(!metric %in% glm_metric)) {
	    stop("for type 'negbin': 'metric' should come from the c('", paste0(glm_metric, collapse = "','"),"').")
	  }
	}
	## check 'tolerance'
	if(!is.numeric(tolerance)) {
		stop("the 'tolerance' must be a numeric value.")
	}else if (tolerance > 1 | tolerance < 0) {
		stop("the 'tolenrance' must be in range 0~1.")
	}
	
	## check 'weights'
	if(!is.null(weight)) {
		if(!is.numeric(weight)) {
			stop("the 'weight' must be a numeric vector.")
		}else if(length(weight) != nrow(data)) {
			stop("the length of the 'weight' vector must equal the number of observation of dataset.")
		}
	}
	
	## check integer of 'best_n'
	if(!is.infinite(best_n)) {
		if(!best_n %% 1 == 0) {
			stop("the 'best_n' must be an integer.")
		}
	}
	
	## check if num_digits:
	if (!is.numeric(num_digits) || num_digits <= 0) {
	  stop("The 'num_digits' must be numeric and above 0!")
	}
}

#' Stepwise Regression Model Selection
#'
#' Performs stepwise regression model selection using various strategies and selection criteria.
#' Supports multiple regression types including linear, logistic, Cox, Poisson, and Gamma regression.
#'
#' @param formula A formula object specifying the model structure:
#'   \itemize{
#'     \item Response variable(s) on left side of ~
#'     \item Predictor variable(s) on right side of ~
#'     \item Use + to separate multiple predictors
#'     \item Use * for main effect and interaction terms
#'     \item Use : for continuous-nested-within-class variable, make sure class variable is a factor variable, e.g. X:A or A:X means a continuous variable X nested within a factor variable A
#'     \item Use . to include all variables
#'     \item Use cbind() for multiple responses
#'     \item Use 0 or -1 to exclude intercept
#'     \item Use strata() to include strata variable for Cox regression
#'   }
#'
#' @param data A data frame containing the variables in the model
#'
#' @param type The type of regression model to fit:
#'   \itemize{
#'     \item "linear" - Linear regression (default)
#'     \item "logit" - Logistic regression
#'     \item "poisson" - Poisson regression
#'     \item "cox" - Cox proportional hazards regression
#'     \item "gamma" - Gamma regression
#'     \item "negbin" - Negative binomial regression
#'   }
#'
#' @param strategy The model selection strategy:
#'   \itemize{
#'     \item "forward" - Forward selection (default)
#'     \item "backward" - Backward elimination
#'     \item "bidirection" - Bidirectional elimination
#'     \item "subset" - Best subset selection
#'   }
#'
#' @param metric The model selection criterion:
#'   \itemize{
#'     \item "AIC" - Akaike Information Criterion (default)
#'     \item "AICc" - Corrected AIC
#'     \item "BIC" - Bayesian Information Criterion
#'     \item "CP" - Mallows' Cp
#'     \item "HQ" - Hannan-Quinn criterion
#'     \item "adjRsq" - Adjusted R-squared
#'     \item "SL" - Significance Level
#'     \item "SBC" - Schwarz Bayesian Criterion
#'     \item "IC(3/2)" - Information Criterion with penalty 3/2
#'     \item "IC(1)" - Information Criterion with penalty 1
#'   }
#'
#' @param sle Significance Level to Enter (default: 0.15). A predictor must have p-value < sle to enter the model.
#'
#' @param sls Significance Level to Stay (default: 0.15). A predictor must have p-value < sls to remain in the model.
#'
#' @param include Character vector of predictor variables that must be included in all models.
#'
#' @param tolerance Threshold for detecting multicollinearity (default: 1e-07). Lower values are more strict.
#'
#' @param weight Optional numeric vector of observation weights. Values are coerced to [0,1].
#'
#' @param test_method_linear Test method for multivariate linear regression:
#'   \itemize{
#'     \item "Pillai" (default)
#'     \item "Wilks"
#'     \item "Hotelling-Lawley"
#'     \item "Roy"
#'   }
#'   For univariate regression, F-test is used.
#' 
#' @param test_method_glm Test method for GLM models:
#'   \itemize{
#'     \item "Rao" (default)
#'     \item "LRT"
#'   }
#'   Only "Rao" available for subset strategy.
#'
#' @param test_method_cox Test method for Cox regression:
#'   \itemize{
#'     \item "efron" (default)
#'     \item "breslow"
#'     \item "exact"
#'   }
#'
#' @param best_n Maximum number of models to retain for each variable count (default: 3)
#' 
#' @param test_ratio Proportion of the dataset allocated for testing (e.g., 0.3, which means 30\% of the dataset is used for testing), with the remainder reserved for training, enabling train-test validation.
#' 
#' @param feature_ratio Proportion of candidate features sampled uniformly at random during forward selection (default = 1). This randomized selection helps identify the best variables while reducing the risk of overfitting, and is only valid when strategy is "forward".
#' 
#' @param seed Seed for random number generation (default: 123), this is only valid when test_ratio or feature_ratio is specified.
#' 
#' @param num_digits Number of decimal places to round results (default: 6)
#'
#' @return A StepReg class object, which is a structured list containing both the input specifications and the outcomes of the stepwise regression analysis. The key components of this object are detailed below, providing a comprehensive framework for model exploration and validation.
#'   \itemize{
#'     \item \code{argument} A data.frame containing the user-specified settings and parameters used in the analysis, including the initial formula, regression type, selection strategy, chosen metrics, significance levels (sle/sls), tolerance threshold, test method, and other control parameters.
#'     \item \code{variable} A data.frame containing information about all variables in the model, including variable names, data types (numeric, factor, etc.), and their roles (Dependent/Independent) in the model.
#'     \item \code{performance} A data.frame providing detailed performance metrics for the selected models across different strategies and metrics. For both training and test datasets (when test_ratio < 1), the output includes model-specific performance indicators:
#'       \itemize{
#'         \item \strong{For linear, poisson, gamma, and negative binomial regression:}
#'           \itemize{
#'             \item \code{adj_r2_train/adj_r2_test}: Adjusted R-squared measures the proportion of variance explained by the model, adjusted for the number of predictors. Values range from 0 to 1, with higher values indicating better model fit. A good model should have high adjusted R-squared on both training and test data, with minimal difference between them. Large differences suggest overfitting.
#'             \item \code{mse_train/mse_test}: Mean Squared Error measures the average squared difference between predicted and actual values. Lower values indicate better model performance. The test MSE should be close to training MSE; significantly higher test MSE suggests overfitting.
#'             \item \code{mae_train/mae_test}: Mean Absolute Error measures the average absolute difference between predicted and actual values. Lower values indicate better model performance. Like MSE, test MAE should be close to training MAE to avoid overfitting.
#'           }
#'         \item \strong{For logistic regression:}
#'           \itemize{
#'             \item \code{accuracy_train/accuracy_test}: Accuracy measures the proportion of correct predictions (true positives + true negatives) / total predictions. Values range from 0 to 1, with higher values indicating better classification performance. Test accuracy should be close to training accuracy; large differences suggest overfitting.
#'             \item \code{auc_train/auc_test}: Area Under the Curve measures the model's ability to distinguish between classes. Values range from 0.5 (random) to 1.0 (perfect discrimination). AUC > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test AUC should be close to training AUC to avoid overfitting.
#'             \item \code{log_loss_train/log_loss_test}: Log Loss (logarithmic loss) penalizes confident wrong predictions more heavily. Lower values indicate better model performance. Values close to 0 are ideal. Test log loss should be close to training log loss; higher test log loss suggests overfitting.
#'           }
#'         \item \strong{For Cox regression:}
#'           \itemize{
#'             \item \code{c-index_train/c-index_test}: Concordance Index (C-index) measures the model's ability to correctly rank survival times. Values range from 0.5 (random) to 1.0 (perfect ranking). C-index > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test C-index should be close to training C-index to avoid overfitting.
#'             \item \code{auc_hc}: Harrell's C-index for time-dependent AUC, measuring discrimination at specific time points. Higher values indicate better discrimination ability.
#'             \item \code{auc_uno}: Uno's C-index for time-dependent AUC, providing an alternative measure of discrimination that may be more robust to censoring patterns.
#'             \item \code{auc_sh}: Schemper and Henderson's C-index for time-dependent AUC, offering another perspective on model discrimination performance.
#'           }
#'       }
#'     \item \code{overview} A nested list organized by strategy and metric, containing step-by-step summaries of the model-building process. Each element shows which variables were entered or removed at each step along with the corresponding metric values (e.g., AIC, BIC, SBC).
#'     \item \code{detail} A nested list organized by strategy and metric, providing granular information about each candidate step. This includes which variables were tested, their evaluation statistics, p-values, and whether they were ultimately selected or rejected.
#'     \item \code{fitted model object within the strategy-specific list} A nested list object organized with a first layer representing the selection strategy (e.g., forward, backward, bidirection, subset) and a second layer representing the metric (e.g., AIC, BIC, SBC). For each strategy-metric combination, the function returns fitted model objects that can be further analyzed using S3 generic functions such as \code{summary()}, \code{anova()}, or \code{coefficients()}. These functions adapt to the model type (e.g., \code{coxph}, \code{lm}, \code{glm}) through call-specific methods. Specific statistics can be directly retrieved using the \code{$} operator, such as \code{result$forward$AIC$coefficients}. The level of detail in these analyses depends on the model type: the \CRANpkg{survival} package enriches \code{coxph} objects with detailed statistics including hazard ratios, standard errors, z-statistics, p-values, and likelihood ratio tests, while base R functions like \code{lm} and \code{glm} offer basic output with coefficients by default, requiring \code{summary()} or \code{anova()} to reveal standard errors, t-values, p-values, and R-squared values.
#'   }
#'
#' @examples
#' # Multivariate linear regression with bidirectional selection
#' data(mtcars)
#' formula <- cbind(mpg, drat) ~ . + 0
#' result1 <- stepwise(
#'   formula = formula,
#'   data = mtcars,
#'   type = "linear",
#'   strategy = "bidirection",
#'   metric = "AIC"
#' )
#' 
#' summary(result1$bidirection$AIC)
#' anova(result1$bidirection$AIC)
#' coefficients(result1$bidirection$AIC)
#'
#' # Linear regression with multiple strategies and metrics
#' formula <- mpg ~ . + 1
#' result2 <- stepwise(
#'   formula = formula,
#'   data = mtcars,
#'   type = "linear",
#'   strategy = c("forward", "bidirection"),
#'   metric = c("AIC", "SBC", "SL", "AICc", "BIC", "HQ")
#' )
#' 
#' summary(result2$forward$AIC)
#' anova(result2$forward$AIC)
#' coefficients(result2$forward$AIC)
#' 
#' # Logistic regression with significance level criteria
#' data(remission)
#' formula <- remiss ~ .
#' result3 <- stepwise(
#'   formula = formula,
#'   data = remission,
#'   type = "logit",
#'   strategy = "forward",
#'   metric = "SL",
#'   sle = 0.05,
#'   sls = 0.05
#' )
#' 
#' summary(result3$forward$SL)
#' anova(result3$forward$SL)
#' coefficients(result3$forward$SL)
#' 
#' # Linear regression with continuous-nested-within-class effects
#' mtcars$am <- factor(mtcars$am)
#' formula <- mpg ~ am + cyl + wt:am + disp:am + hp:am
#' result4 <- stepwise(
#'   formula = formula,
#'   data = mtcars,
#'   type = "linear",
#'   strategy = "bidirection",
#'   metric = "AIC"
#' )
#' 
#' summary(result4$bidirection$AIC)
#' anova(result4$bidirection$AIC)
#' coefficients(result4$bidirection$AIC)
#' 
#' @references
#' \itemize{
#'   \item Alsubaihi et al. (2002) Variable strategy in multivariable regression using sas/iml
#'   \item Darlington (1968) Multiple regression in psychological research and practice
#'   \item Dharmawansa et al. (2014) Roy's largest root under rank-one alternatives
#'   \item Hannan & Quinn (1979) The determination of the order of an autoregression
#'   \item Hotelling (1992) The Generalization of Student's Ratio
#'   \item Hocking (1976) The analysis and strategy of variables in linear regression
#'   \item Hurvich & Tsai (1989) Regression and time series model strategy in small samples
#'   \item Judge (1985) The Theory and practice of econometrics
#'   \item Mallows (1973) Some comments on cp
#'   \item Mardia et al. (1979) Multivariate analysis
#'   \item Mckeon (1974) F approximations to the distribution of hotelling's t20
#'   \item Mcquarrie & Tsai (1998) Regression and Time Series Model strategy
#'   \item Pillai (1955) Some new test criteria in multivariate analysis
#'   \item Sparks et al. (1985) On variable strategy in multivariate regression
#'   \item Sawa (1978) Information criteria for discriminating among alternative regression models
#'   \item Schwarz (1978) Estimating the dimension of a model
#' }
#'
#' @author Junhui Li, Kai Hu, Xiaohuan Lu
#'
#' @keywords stepwise regression
#'
#' @importFrom survival coxph concordance Surv
#' @importFrom survAUC AUC.uno AUC.sh AUC.hc
#' @importFrom utils combn
#' @importFrom pROC auc roc
#' @importFrom stats anova coef glm lm logLik pf reformulate sigma terms deviance df.residual formula model.frame predict cor
#' @importFrom MASS glm.nb
#'
#' @export

stepwise <- function(formula,
                     data,
                     type = c("linear", "logit", "cox", "poisson", "gamma", "negbin"),
                     strategy = c("forward", "backward", "bidirection", "subset"),
                     metric = c("AIC", "AICc", "BIC", "CP", "HQ", "adjRsq", "SL", "SBC", "IC(3/2)", "IC(1)"),
                     sle = 0.15,
                     sls = 0.15,
                     include = NULL,
                     test_method_linear = c("Pillai", "Wilks", "Hotelling-Lawley", "Roy"),
                     test_method_glm = c("Rao", "LRT"),
                     test_method_cox = c("efron", "breslow", "exact"),
                     tolerance = 1e-7,
                     weight = NULL,
                     best_n = 3,
                     test_ratio = 0,
                     feature_ratio = 1,
                     seed = 123,
                     num_digits = 6) {
  type <- match.arg(type)
  strategy <- match_multiple_args(strategy, c("forward", "backward", "bidirection", "subset"))
  metric <- match_multiple_args(metric, c("AIC", "AICc", "BIC", "CP", "HQ", "adjRsq", "SL", "SBC", "IC(3/2)", "IC(1)"))
  
  test_method_linear <- match.arg(test_method_linear)
  test_method_glm <- match.arg(test_method_glm)
  test_method_cox <- match.arg(test_method_cox)

  if(test_ratio >= 1 | test_ratio < 0) {
    stop("test_ratio must be between 0 and 1")
  }
  if(feature_ratio > 1 | feature_ratio <= 0) {
    stop("feature_ratio must be between 0 and 1")
  }
  if(feature_ratio < 1) {
    set.seed(seed)
  }
  data_train <- data
  data_test <- NULL
  if(test_ratio > 0) {
    set.seed(seed)
    data_test <- data[sample(1:nrow(data), size = round(test_ratio * nrow(data), 0)), ]
    data_train <- data[sample(1:nrow(data), size = round((1 - test_ratio) * nrow(data), 0)), ]
  }

  x_name_orig <- getXname(formula, data_train)
  y_name <- getYname(formula, data_train)
  intercept <- getIntercept(formula, data_train, type = type) # char type
  merged_include <- getMergedVar(include)
  model_raw <- getModel(data_train, type = type, intercept = intercept, x_name_orig, y_name, weight = weight, method = test_method_cox)
  if(type != "cox") {
    y_df <- as.matrix(model_raw$model[, y_name])
    n_y <- ncol(y_df)
  }else{
    n_y <- 1
  }
  sigma_value <- getSigmaFullModel(model_raw, type, n_y)
  validateUtils(formula = formula, data = data_train, type = type, include = include, strategy = strategy, metric = metric, sle = sle, sls = sls, sigma_value = sigma_value, test_method_linear = test_method_linear, test_method_glm = test_method_glm, test_method_cox = test_method_cox, tolerance = tolerance, weight = weight, best_n = best_n, n_y = n_y)
  test_method <- getTestMethod(data_train, model_raw, type, metric, n_y, test_method_linear, test_method_glm, test_method_cox)
  
  multico_x <- getMulticolX(data_train, x_name_orig, tolerance)
  merged_multico_x <- getMergedVar(multico_x)
  x_name <- setdiff(x_name_orig, multico_x)

  dummy_name_list <- dummyNameList(x_name, data_train)
  
  result <- list()
  ## table1
  table1_para_value <- getTable1SummaryOfParameters(formula, data_train, type, x_name_orig, y_name, merged_multico_x, merged_include, strategy, metric, sle, sls, test_method, tolerance, intercept, test_ratio, feature_ratio, seed)
  result$argument <- table1_para_value
  
  ## table2
  table2_class_table <- getTable2TypeOfVariables(model_raw)
  result$variable <- table2_class_table
  
  ## table3
  table3_process <- getTable3ProcessSummary(data_train=data_train, data_test=data_test, type, strategy, metric, sle, sls, weight, x_name, y_name, intercept, include, best_n, test_method, sigma_value, num_digits, feature_ratio, dummy_name_list)
  x_final_model_metric <- table3_process$final_variable
  result <- append(result,table3_process[which(names(table3_process) != "final_variable")])
  
  ## table4
  table4_model <- getTable4ModelCall(type, intercept, include, x_final_model_metric, y_name, n_y, data_train, weight, test_method, num_digits)
  result <- append(result,table4_model)

  class(result) <- c("StepReg","list")
  attr(result, "nonhidden") <- strategy
  return(result)
}

Sys.setenv(R_TESTS="")
test_that("test_stepwise.R failed", {
  linear_model1 <- mpg ~ . + 1
  linear_model2 <- cbind(mpg, drat) ~ . + 0
  logit_model1 <- remiss ~ .
  cox_model1 <- Surv(time, status) ~ .
  ## the object should be located in inst/extdata instead of tests/data, since When you run devtools::check(), the package is installed in a temporary directory, tests/data is not in this temporary dir.
  ## If you have data files that need to be accessed during package checks, it's a good practice to place them in the inst/extdata directory of your package. Files in this directory are installed with the package and can be accessed using system.file().
  ## When you run devtools::check(), the package is installed in a temporary directory, and the working directory may not be set to the package directory itself. The system.file() function in R looks for files relative to the package installation directory, and if the working directory is not set correctly during the check process, it may lead to issues.
  res_v1_5_0 <- readRDS(system.file("extdata","res_v1_5_0.rds", package = "StepReg"))
  for (mod in names(res_v1_5_0)){
    type <- unlist(strsplit(mod,"_"))[1]
    if(mod=="cox_model1"){
      lung <- na.omit(survival::lung)
      mydata <- lung
    }else if(mod %in% c("linear_model1","linear_model2")){
      data(mtcars)
      mtcars$yes <- mtcars$wt
      mydata <- mtcars
    }else if(mod %in% "logit_model1"){
      data(remission)
      mydata <- remission
    }
    #strategy="forward"
    for (strategy in names(res_v1_5_0[[mod]])){
      if(strategy=="subset"){
        index_n <- 2
        select_col1 <- c(1,2,3)
      }else{
        index_n <- 3
        select_col1 <- c(2,3,7)
      }
      #metric="SL"
      for(metric in names(res_v1_5_0[[mod]][[strategy]])){
        message(mod,"\t",strategy,"\t",metric)
        output_new <- NA
        output_old <- res_v1_5_0[[mod]][[strategy]][[metric]][,select_col1]
        
        try(output_new <- stepwise(type = type,
                                    formula=get(mod),
                                    data=mydata,
                                    strategy="backward",
                                    metric="SL",
                                    sle=0.05,
                                    sls=0.05)[[3]],silent = TRUE)
        output_new
        res <- try(expect_equal(output_new,output_old),silent = TRUE)
        if(inherits(res, "try-error")){
          message("Error\t",mod,"\t",strategy,"\t",metric)
        }  
      } #metric
    } #strategy
  } #mod
})

#' Model Performance Summary Across Different Selection Strategies
#'
#' Creates a summary table showing the performance of the selected models by different combinations of 
#' stepwise regression strategies and selection metrics.
#'
#' @param x A list object returned by the \code{stepwise()} function
#' 
#' @param ... Additional arguments (currently not used)
#'
#' @return A data frame where:
#'   \item{model}{The formula of each selected model}
#'   \item{strategy:metric}{Columns for each combination of strategy and metric used}
#'   \itemize{
#'     \item \strong{For linear, poisson, gamma, and negative binomial regression:}
#'       \itemize{
#'         \item \code{adj_r2_train/adj_r2_test}: Adjusted R-squared measures the proportion of variance explained by the model, adjusted for the number of predictors. Values range from 0 to 1, with higher values indicating better model fit. A good model should have high adjusted R-squared on both training and test data, with minimal difference between them. Large differences suggest overfitting.
#'         \item \code{mse_train/mse_test}: Mean Squared Error measures the average squared difference between predicted and actual values. Lower values indicate better model performance. The test MSE should be close to training MSE; significantly higher test MSE suggests overfitting.
#'         \item \code{mae_train/mae_test}: Mean Absolute Error measures the average absolute difference between predicted and actual values. Lower values indicate better model performance. Like MSE, test MAE should be close to training MAE to avoid overfitting.
#'       }
#'     \item \strong{For logistic regression:}
#'       \itemize{
#'         \item \code{accuracy_train/accuracy_test}: Accuracy measures the proportion of correct predictions (true positives + true negatives) / total predictions. Values range from 0 to 1, with higher values indicating better classification performance. Test accuracy should be close to training accuracy; large differences suggest overfitting.
#'         \item \code{auc_train/auc_test}: Area Under the Curve measures the model's ability to distinguish between classes. Values range from 0.5 (random) to 1.0 (perfect discrimination). AUC > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test AUC should be close to training AUC to avoid overfitting.
#'         \item \code{log_loss_train/log_loss_test}: Log Loss (logarithmic loss) penalizes confident wrong predictions more heavily. Lower values indicate better model performance. Values close to 0 are ideal. Test log loss should be close to training log loss; higher test log loss suggests overfitting.
#'       }
#'     \item \strong{For Cox regression:}
#'       \itemize{
#'         \item \code{c-index_train/c-index_test}: Concordance Index (C-index) measures the model's ability to correctly rank survival times. Values range from 0.5 (random) to 1.0 (perfect ranking). C-index > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test C-index should be close to training C-index to avoid overfitting.
#'         \item \code{auc_hc}: Harrell's C-index for time-dependent AUC, measuring discrimination at specific time points. Higher values indicate better discrimination ability.
#'         \item \code{auc_uno}: Uno's C-index for time-dependent AUC, providing an alternative measure of discrimination that may be more robust to censoring patterns.
#'         \item \code{auc_sh}: Schemper and Henderson's C-index for time-dependent AUC, offering another perspective on model discrimination performance.
#'       }
#'   }
#'   
  #' Each cell contains the performance of the model by the corresponding 
  #' strategy-metric combination. For the subset strategy with Information Criteria (IC), 
  #' only the single best model across all variable numbers is shown. This does not apply 
  #' to Significance Level (SL) since F/Rao statistics can only be compared between models 
  #' with the same number of variables.
#'
#' @examples
#' # Load example data
#' data(mtcars)
#' 
#' # Run stepwise regression with multiple strategies and metrics
#' formula <- mpg ~ .
#' result <- stepwise(
#'   formula = formula,
#'   data = mtcars,
#'   type = "linear",
#'   strategy = c("forward", "backward", "bidirection"),
#'   metric = c("AIC", "BIC")
#' )
#' 
#' # Get performance summary
#' performance(result)
#'
#' @export
#' 
performance <- function(x, ...){
  performance_df <- x[["performance"]]
  if("response" %in% colnames(performance_df)) {
    performance_df$model <- paste(performance_df$response, performance_df$model, sep=":")
  }
  performance_mat <- as.data.frame(performance_df)
  
  uniq_model <- unique(performance_mat[,1])
  performance_df <- as.data.frame(matrix("",length(uniq_model),ncol(performance_mat)))
  performance_df[,1] <- uniq_model
  for(i in 1:length(uniq_model)) {
    performance_df[i,2] <- paste(performance_mat[performance_mat[,"model"] %in% uniq_model[i],2], collapse = "; ")
    performance_df[i,3:ncol(performance_mat)] <- performance_mat[which(performance_mat[,"model"] %in% uniq_model[i])[1],3:ncol(performance_mat)]
  }
  colnames(performance_df) <- colnames(performance_mat)
  if("response" %in% colnames(performance_df)) {
    performance_df$model <- sub(".*cbind\\(", "cbind(", performance_df$model)
    last_col <- ncol(performance_df) - 1
  } else {
    last_col <- ncol(performance_df)
  }
  numeric_cols <- colnames(performance_df)[3:last_col]
  
  performance_df[, numeric_cols] <- as.data.frame(
    lapply(performance_df[, numeric_cols], as.numeric)
  )
  performance_df[, numeric_cols] <- round(performance_df[, numeric_cols], digits = 4)
  #class(vote_reform) <- c("StepReg")
  return(performance_df)
}

---
title: "StepReg"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{StepReg}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(StepReg)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tobacco.R
\docType{data}
\name{tobacco}
\alias{tobacco}
\title{Tobacco Leaf Chemical Composition Dataset}
\format{
A data frame with 25 rows and 9 variables:
\describe{
  \item{cigarette}{Numeric. Rate of cigarette burn in inches per 1000 seconds.}
  \item{sugar}{Numeric. Percentage of sugar content in the leaf.}
  \item{nicotine}{Numeric. Percentage of nicotine content.}
  \item{nitrogen}{Numeric. Percentage of nitrogen content.}
  \item{chlorine}{Numeric. Percentage of chlorine content.}
  \item{potassium}{Numeric. Percentage of potassium content.}
  \item{phosphorus}{Numeric. Percentage of phosphorus content.}
  \item{calcium}{Numeric. Percentage of calcium content.}
  \item{magnesium}{Numeric. Percentage of magnesium content.}
}
}
\source{
Anderson, R. L. and Bancroft, T. A. (1952), Statistical Theory in Research,
McGraw-Hill Book Company, Inc., New York, NY.
}
\usage{
data(tobacco)
}
\description{
A dataset containing chemical composition measurements from 25 tobacco leaf samples.
This dataset is commonly used for demonstrating multivariate regression analysis
and exploring relationships between chemical components and burn rate.
}
\details{
This dataset is particularly useful for:
\itemize{
  \item Demonstrating multivariate regression analysis
  \item Studying relationships between chemical composition and burn rate
  \item Analyzing correlations between different chemical components
  \item Teaching statistical methods in agricultural research
}
}
\examples{
# Load the dataset
data(tobacco)

# View the first few rows
head(tobacco)

# Summary statistics
summary(tobacco)

# Correlation analysis
cor(tobacco)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.StepReg.R
\name{plot.StepReg}
\alias{plot.StepReg}
\title{Visualize Stepwise Regression Results}
\usage{
\method{plot}{StepReg}(
  x,
  strategy = attr(x, "nonhidden"),
  process = c("overview", "detail"),
  num_digits = 6,
  ...
)
}
\arguments{
\item{x}{A StepReg object containing the results of stepwise regression analysis.}

\item{strategy}{Character. Specifies which selection strategy to visualize:
\itemize{
  \item "forward" - Forward selection
  \item "backward" - Backward elimination
  \item "bidirection" - Bidirectional selection
  \item "subset" - Best subset selection
}
Default is the first strategy name in the StepReg object.}

\item{process}{Character. Specifies the type of visualization to display:
\itemize{
  \item "detail" - Shows detailed step-by-step selection process with variable entry/removal
  \item "overview" - Shows summary of the selection process with metric values
}
Default is "overview".}

\item{num_digits}{Integer. Number of decimal places to display in the plots.
Default is 6.}

\item{...}{Additional argument passed to plotting functions (currently not used).}
}
\value{
A ggplot object showing either:
  \itemize{
    \item For "detail" process: A heatmap showing variable selection status at each step
    \item For "overview" process: A line plot showing metric values across steps
  }
}
\description{
Creates informative visualizations of the variable selection process from a StepReg object.
This function generates two types of plots: detailed step-by-step selection process and
an overview of the final selected variables.
}
\details{
The function creates different types of visualizations based on the selection strategy:
  \itemize{
    \item For forward/backward/bidirectional selection:
      \itemize{
        \item detail view shows a heatmap with green tiles for added variables,
              tan tiles for removed variables, and gray tiles for non-selected variables
        \item Overview shows metric values across steps with variable labels
      }
    \item For subset selection:
      \itemize{
        \item detail view shows a heatmap of selected variables at each step
        \item Overview shows metric values for different subset sizes
      }
  }
}
\examples{
\dontrun{
# Load example data
data(mtcars)

# Run stepwise regression with multiple strategies
formula <- mpg ~ .
result <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = c("forward", "bidirection", "subset"),
  metric = c("AIC", "BIC", "SL")
)

# Generate default overview plot
plot(result)

# Generate detailed plot for forward selection
plot(result, strategy = "forward", process = "detail")

# Generate overview plot with 3 decimal places
plot(result, strategy = "bidirection", process = "overview", num_digits = 3)
}

}
\seealso{
\code{\link{stepwise}} for creating StepReg objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StepReg-package.R
\docType{package}
\name{StepReg-package}
\alias{StepReg}
\alias{StepReg-package}
\title{StepReg: Stepwise Regression Analysis}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Stepwise regression is a statistical technique used for model selection. This package streamlines stepwise regression analysis by supporting multiple regression types(linear, Cox, logistic, Poisson, Gamma, and negative binomial), incorporating popular selection strategies(forward, backward, bidirectional, and subset), and offering essential metrics. It enables users to apply multiple selection strategies and metrics in a single function call, visualize variable selection processes, and export results in various formats. StepReg offers a data-splitting option to address potential issues with invalid statistical inference and a randomized forward selection option to avoid overfitting. We validated StepReg's accuracy using public datasets within the SAS software environment. For an interactive web interface, users can install the companion 'StepRegShiny' package.
}
\seealso{
Useful links:
\itemize{
  \item Report bugs at \url{https://github.com/JunhuiLi1017/StepReg/issues}
}

}
\author{
\strong{Maintainer}: Junhui Li \email{junhui.li11@umassmed.edu} (\href{https://orcid.org/0000-0003-3973-1700}{ORCID})

Authors:
\itemize{
  \item Junhui Li \email{junhui.li11@umassmed.edu}
  \item Kai Hu \email{kai.hu@umassmed.edu}
  \item Xiaohuan Lu
  \item Wenxin Liu
  \item Lihua Julie Zhu
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remission.R
\docType{data}
\name{remission}
\alias{remission}
\title{Leukemia Remission Dataset}
\format{
A data frame with 27 observations and 7 variables:
\describe{
  \item{remiss}{Binary outcome variable indicating leukemia remission status:
    \itemize{
      \item 1 = Remission occurred
      \item 0 = No remission
    }
  }
  \item{cell}{Numeric. Cellularity of the marrow clot section (percentage)}
  \item{smear}{Numeric. Smear differential percentage of blasts}
  \item{infil}{Numeric. Percentage of absolute marrow leukemia cell infiltrate}
  \item{li}{Numeric. Percentage labeling index of the bone marrow leukemia cells}
  \item{blast}{Numeric. Absolute number of blasts in the peripheral blood}
  \item{temp}{Numeric. Highest temperature (in Fahrenheit) before treatment}
}
}
\source{
Lee, E. T. (1974). "A Computer Program for Linear Logistic Regression Analysis."
Computer Programs in Biomedicine 4:80–92.
}
\usage{
data(remission)
}
\description{
A dataset containing information about leukemia remission and associated risk factors.
This dataset is commonly used for demonstrating logistic regression analysis in medical research.
}
\details{
This dataset is particularly useful for:
\itemize{
  \item Demonstrating logistic regression analysis
  \item Studying risk factors for leukemia remission
  \item Teaching medical statistics and predictive modeling
}
}
\examples{
\dontrun{
# Load the dataset
data(remission)

# View first few rows
head(remission)

# Summary statistics
summary(remission)

# Run logistic regression
model <- glm(remiss ~ ., data = remission, family = binomial)
summary(model)
}

}
\references{
\itemize{
  \item Lee, E. T. (1974). "A Computer Program for Linear Logistic Regression Analysis."
  Computer Programs in Biomedicine 4:80–92.
  \item Penn State University Statistics Online. "Logistic Regression Example."
  \url{https://online.stat.psu.edu/stat501/book/export/html/1011}
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/creditCard.R
\docType{data}
\name{creditCard}
\alias{creditCard}
\title{Credit Card Application Dataset}
\format{
A data frame with 1,319 observations and 12 variables:
\describe{
  \item{card}{Factor. Whether the credit card application was accepted (Yes/No)}
  \item{reports}{Numeric. Number of major derogatory reports on the applicant's credit history}
  \item{age}{Numeric. Age in years plus twelfths of a year (e.g., 30.5 represents 30 years and 6 months)}
  \item{income}{Numeric. Annual income in USD (divided by 10,000)}
  \item{share}{Numeric. Ratio of monthly credit card expenditure to yearly income}
  \item{expenditure}{Numeric. Average monthly credit card expenditure in USD}
  \item{owner}{Factor. Home ownership status (Yes/No)}
  \item{selfemp}{Factor. Self-employment status (Yes/No)}
  \item{dependents}{Numeric. Number of dependents}
  \item{months}{Numeric. Number of months living at current address}
  \item{majorcards}{Numeric. Number of major credit cards held}
  \item{active}{Numeric. Number of active credit accounts}
}
}
\source{
Greene, W.H. (2003). Econometric Analysis, 5th edition. Upper Saddle River, NJ: Prentice Hall.
}
\usage{
data(creditCard)
}
\description{
A dataset containing credit history information for credit card applicants. This dataset is sourced from the AER package.
}
\details{
This dataset is commonly used for credit risk analysis and modeling credit card approval decisions. 
It provides a comprehensive view of various factors that may influence credit card application outcomes.
}
\examples{
data(creditCard)
summary(creditCard)
}
\seealso{
\code{\link[AER]{CreditCard}} for the original dataset in the AER package
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/affairs.R
\docType{data}
\name{affairs}
\alias{affairs}
\title{Fair's Extramarital Affairs Data}
\format{
A data frame containing 601 observations on 9 variables.
\describe{
  \item{affairs}{numeric. How often engaged in extramarital sexual intercourse during the past year? 0 = none, 1 = once, 2 = twice, 3 = 3 times, 7 = 4–10 times, 12 = monthly, 12 = weekly, 12 = daily.}
  \item{gender}{factor indicating gender.}
  \item{age}{numeric variable coding age in years: 17.5 = under 20, 22 = 20–24, 27 = 25–29, 32 = 30–34, 37 = 35–39, 42 = 40–44, 47 = 45–49, 52 = 50–54, 57 = 55 or over.}
  \item{yearsmarried}{numeric variable coding number of years married: 0.125 = 3 months or less, 0.417 = 4–6 months, 0.75 = 6 months–1 year, 1.5 = 1–2 years, 4 = 3–5 years, 7 = 6–8 years, 10 = 9–11 years, 15 = 12 or more years.}
  \item{children}{factor. Are there children in the marriage?}
  \item{religiousness}{numeric variable coding religiousness: 1 = anti, 2 = not at all, 3 = slightly, 4 = somewhat, 5 = very.}
  \item{education}{numeric variable coding level of education: 9 = grade school, 12 = high school graduate, 14 = some college, 16 = college graduate, 17 = some graduate work, 18 = master's degree, 20 = Ph.D., M.D., or other advanced degree.}
  \item{occupation}{numeric variable coding occupation according to Hollingshead classification (reverse numbering).}
  \item{rating}{numeric variable coding self rating of marriage: 1 = very unhappy, 2 = somewhat unhappy, 3 = average, 4 = happier than average, 5 = very happy.}
}
}
\source{
Fair, R.C. (1978). A Theory of Extramarital Affairs. Journal of Political Economy, 86, 45–61.
}
\usage{
data(affairs)
}
\description{
Infidelity data, known as Fair's Affairs. Cross-section data from a survey conducted by Psychology Today in 1969. This dataset is sourced from the AER package.
}
\examples{
data(affairs)
summary(affairs)
}
\references{
\itemize{
  \item Greene, W.H. (2003). Econometric Analysis, 5th edition. Upper Saddle River, NJ: Prentice Hall.

\item Fair, R.C. (1978). A Theory of Extramarital Affairs. Journal of Political Economy, 86, 45–61.
}
}
\seealso{
\code{\link[AER]{Affairs}} for the original dataset in the AER package
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/performance.R
\name{performance}
\alias{performance}
\title{Model Performance Summary Across Different Selection Strategies}
\usage{
performance(x, ...)
}
\arguments{
\item{x}{A list object returned by the \code{stepwise()} function}

\item{...}{Additional arguments (currently not used)}
}
\value{
A data frame where:
  \item{model}{The formula of each selected model}
  \item{strategy:metric}{Columns for each combination of strategy and metric used}
  \itemize{
    \item \strong{For linear, poisson, gamma, and negative binomial regression:}
      \itemize{
        \item \code{adj_r2_train/adj_r2_test}: Adjusted R-squared measures the proportion of variance explained by the model, adjusted for the number of predictors. Values range from 0 to 1, with higher values indicating better model fit. A good model should have high adjusted R-squared on both training and test data, with minimal difference between them. Large differences suggest overfitting.
        \item \code{mse_train/mse_test}: Mean Squared Error measures the average squared difference between predicted and actual values. Lower values indicate better model performance. The test MSE should be close to training MSE; significantly higher test MSE suggests overfitting.
        \item \code{mae_train/mae_test}: Mean Absolute Error measures the average absolute difference between predicted and actual values. Lower values indicate better model performance. Like MSE, test MAE should be close to training MAE to avoid overfitting.
      }
    \item \strong{For logistic regression:}
      \itemize{
        \item \code{accuracy_train/accuracy_test}: Accuracy measures the proportion of correct predictions (true positives + true negatives) / total predictions. Values range from 0 to 1, with higher values indicating better classification performance. Test accuracy should be close to training accuracy; large differences suggest overfitting.
        \item \code{auc_train/auc_test}: Area Under the Curve measures the model's ability to distinguish between classes. Values range from 0.5 (random) to 1.0 (perfect discrimination). AUC > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test AUC should be close to training AUC to avoid overfitting.
        \item \code{log_loss_train/log_loss_test}: Log Loss (logarithmic loss) penalizes confident wrong predictions more heavily. Lower values indicate better model performance. Values close to 0 are ideal. Test log loss should be close to training log loss; higher test log loss suggests overfitting.
      }
    \item \strong{For Cox regression:}
      \itemize{
        \item \code{c-index_train/c-index_test}: Concordance Index (C-index) measures the model's ability to correctly rank survival times. Values range from 0.5 (random) to 1.0 (perfect ranking). C-index > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test C-index should be close to training C-index to avoid overfitting.
        \item \code{auc_hc}: Harrell's C-index for time-dependent AUC, measuring discrimination at specific time points. Higher values indicate better discrimination ability.
        \item \code{auc_uno}: Uno's C-index for time-dependent AUC, providing an alternative measure of discrimination that may be more robust to censoring patterns.
        \item \code{auc_sh}: Schemper and Henderson's C-index for time-dependent AUC, offering another perspective on model discrimination performance.
      }
  }
  
Each cell contains the performance of the model by the corresponding 
strategy-metric combination. For the subset strategy with Information Criteria (IC), 
only the single best model across all variable numbers is shown. This does not apply 
to Significance Level (SL) since F/Rao statistics can only be compared between models 
with the same number of variables.
}
\description{
Creates a summary table showing the performance of the selected models by different combinations of 
stepwise regression strategies and selection metrics.
}
\examples{
# Load example data
data(mtcars)

# Run stepwise regression with multiple strategies and metrics
formula <- mpg ~ .
result <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = c("forward", "backward", "bidirection"),
  metric = c("AIC", "BIC")
)

# Get performance summary
performance(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lung.R
\docType{data}
\name{lung}
\alias{lung}
\title{NCCTG Lung Cancer Data}
\format{
A data frame containing 228 observations on 10 variables.
\describe{
  \item{inst}{Institution code}
  \item{time}{Survival time in days}
  \item{status}{censoring status 1=censored, 2=dead}
  \item{age}{Age in years}
  \item{sex}{Male=1 Female=2}
  \item{ph.ecog}{ECOG performance score as rated by the physician. 0=asymptomatic, 1= symptomatic but completely ambulatory, 2= in bed < 50\% of the day, 3= in bed > 50\% of the day but not bedbound, 4 = bedbound}
  \item{ph.karno}{Karnofsky performance score (bad=0-good=100) rated by physician}
  \item{pat.karno}{Karnofsky performance score as rated by patient}
  \item{meal.cal}{Calories consumed at meals}
  \item{wt.loss}{Weight loss in last six months (pounds)}
}
}
\source{
Terry Therneau. (2021). survival: Survival Analysis. R package version 3.4-2. \url{https://CRAN.R-project.org/package=survival}
}
\usage{
data(lung)
}
\description{
Survival in patients with advanced lung cancer from the North Central Cancer Treatment Group. Performance scores rate how well the patient can perform usual daily activities.
}
\note{
This dataset is sourced from the survival package.
}
\examples{
data(lung)
summary(lung)
}
\references{
\itemize{
  \item Loprinzi CL. Laurie JA. Wieand HS. Krook JE. Novotny PJ. Kugler JW. Bartel J. Law M. Bateman M. Klatt NE. et al. Prospective evaluation of prognostic variables from patient-completed questionnaires. North Central Cancer Treatment Group. Journal of Clinical Oncology. 12(3):601-7, 1994.
}
}
\seealso{
\code{\link[survival]{lung}} for the original dataset in the survival package
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.StepReg.R
\name{print.StepReg}
\alias{print.StepReg}
\title{Print Stepwise Regression Results}
\usage{
\method{print}{StepReg}(x, ...)
}
\arguments{
\item{x}{A StepReg object containing the results of stepwise regression analysis.}

\item{...}{Additional arguments passed to the print method (currently not used).}
}
\value{
Invisibly returns the printed object. The function displays:
  \itemize{
    \item Final model fit statistics for each strategy and metric combination
  }
}
\description{
Displays the final model fit statistics from a StepReg object. This function provides
a concise summary of the selected model's performance metrics.
}
\details{
The print method provides a focused view of the final model's performance,
showing the selected variables and their corresponding fit statistics. This is useful
for quickly assessing the model's quality without the detailed step-by-step selection
process (which can be viewed using \code{\link{stepwise}}).
}
\examples{
\dontrun{
# Load example data
data(mtcars)

# Run stepwise regression
formula <- mpg ~ .
result <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = "forward",
  metric = "AIC"
)

# Print final model statistics
result
}

}
\seealso{
\code{\link{stepwise}} for creating StepReg objects

\code{\link{plot.StepReg}} for visualization of results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stepwise.R
\name{stepwise}
\alias{stepwise}
\title{Stepwise Regression Model Selection}
\usage{
stepwise(
  formula,
  data,
  type = c("linear", "logit", "cox", "poisson", "gamma", "negbin"),
  strategy = c("forward", "backward", "bidirection", "subset"),
  metric = c("AIC", "AICc", "BIC", "CP", "HQ", "adjRsq", "SL", "SBC", "IC(3/2)", "IC(1)"),
  sle = 0.15,
  sls = 0.15,
  include = NULL,
  test_method_linear = c("Pillai", "Wilks", "Hotelling-Lawley", "Roy"),
  test_method_glm = c("Rao", "LRT"),
  test_method_cox = c("efron", "breslow", "exact"),
  tolerance = 1e-07,
  weight = NULL,
  best_n = 3,
  test_ratio = 0,
  feature_ratio = 1,
  seed = 123,
  num_digits = 6
)
}
\arguments{
\item{formula}{A formula object specifying the model structure:
\itemize{
  \item Response variable(s) on left side of ~
  \item Predictor variable(s) on right side of ~
  \item Use + to separate multiple predictors
  \item Use * for main effect and interaction terms
  \item Use : for continuous-nested-within-class variable, make sure class variable is a factor variable, e.g. X:A or A:X means a continuous variable X nested within a factor variable A
  \item Use . to include all variables
  \item Use cbind() for multiple responses
  \item Use 0 or -1 to exclude intercept
  \item Use strata() to include strata variable for Cox regression
}}

\item{data}{A data frame containing the variables in the model}

\item{type}{The type of regression model to fit:
\itemize{
  \item "linear" - Linear regression (default)
  \item "logit" - Logistic regression
  \item "poisson" - Poisson regression
  \item "cox" - Cox proportional hazards regression
  \item "gamma" - Gamma regression
  \item "negbin" - Negative binomial regression
}}

\item{strategy}{The model selection strategy:
\itemize{
  \item "forward" - Forward selection (default)
  \item "backward" - Backward elimination
  \item "bidirection" - Bidirectional elimination
  \item "subset" - Best subset selection
}}

\item{metric}{The model selection criterion:
\itemize{
  \item "AIC" - Akaike Information Criterion (default)
  \item "AICc" - Corrected AIC
  \item "BIC" - Bayesian Information Criterion
  \item "CP" - Mallows' Cp
  \item "HQ" - Hannan-Quinn criterion
  \item "adjRsq" - Adjusted R-squared
  \item "SL" - Significance Level
  \item "SBC" - Schwarz Bayesian Criterion
  \item "IC(3/2)" - Information Criterion with penalty 3/2
  \item "IC(1)" - Information Criterion with penalty 1
}}

\item{sle}{Significance Level to Enter (default: 0.15). A predictor must have p-value < sle to enter the model.}

\item{sls}{Significance Level to Stay (default: 0.15). A predictor must have p-value < sls to remain in the model.}

\item{include}{Character vector of predictor variables that must be included in all models.}

\item{test_method_linear}{Test method for multivariate linear regression:
\itemize{
  \item "Pillai" (default)
  \item "Wilks"
  \item "Hotelling-Lawley"
  \item "Roy"
}
For univariate regression, F-test is used.}

\item{test_method_glm}{Test method for GLM models:
\itemize{
  \item "Rao" (default)
  \item "LRT"
}
Only "Rao" available for subset strategy.}

\item{test_method_cox}{Test method for Cox regression:
\itemize{
  \item "efron" (default)
  \item "breslow"
  \item "exact"
}}

\item{tolerance}{Threshold for detecting multicollinearity (default: 1e-07). Lower values are more strict.}

\item{weight}{Optional numeric vector of observation weights. Values are coerced to [0,1].}

\item{best_n}{Maximum number of models to retain for each variable count (default: 3)}

\item{test_ratio}{Proportion of the dataset allocated for testing (e.g., 0.3, which means 30\% of the dataset is used for testing), with the remainder reserved for training, enabling train-test validation.}

\item{feature_ratio}{Proportion of candidate features sampled uniformly at random during forward selection (default = 1). This randomized selection helps identify the best variables while reducing the risk of overfitting, and is only valid when strategy is "forward".}

\item{seed}{Seed for random number generation (default: 123), this is only valid when test_ratio or feature_ratio is specified.}

\item{num_digits}{Number of decimal places to round results (default: 6)}
}
\value{
A StepReg class object, which is a structured list containing both the input specifications and the outcomes of the stepwise regression analysis. The key components of this object are detailed below, providing a comprehensive framework for model exploration and validation.
  \itemize{
    \item \code{argument} A data.frame containing the user-specified settings and parameters used in the analysis, including the initial formula, regression type, selection strategy, chosen metrics, significance levels (sle/sls), tolerance threshold, test method, and other control parameters.
    \item \code{variable} A data.frame containing information about all variables in the model, including variable names, data types (numeric, factor, etc.), and their roles (Dependent/Independent) in the model.
    \item \code{performance} A data.frame providing detailed performance metrics for the selected models across different strategies and metrics. For both training and test datasets (when test_ratio < 1), the output includes model-specific performance indicators:
      \itemize{
        \item \strong{For linear, poisson, gamma, and negative binomial regression:}
          \itemize{
            \item \code{adj_r2_train/adj_r2_test}: Adjusted R-squared measures the proportion of variance explained by the model, adjusted for the number of predictors. Values range from 0 to 1, with higher values indicating better model fit. A good model should have high adjusted R-squared on both training and test data, with minimal difference between them. Large differences suggest overfitting.
            \item \code{mse_train/mse_test}: Mean Squared Error measures the average squared difference between predicted and actual values. Lower values indicate better model performance. The test MSE should be close to training MSE; significantly higher test MSE suggests overfitting.
            \item \code{mae_train/mae_test}: Mean Absolute Error measures the average absolute difference between predicted and actual values. Lower values indicate better model performance. Like MSE, test MAE should be close to training MAE to avoid overfitting.
          }
        \item \strong{For logistic regression:}
          \itemize{
            \item \code{accuracy_train/accuracy_test}: Accuracy measures the proportion of correct predictions (true positives + true negatives) / total predictions. Values range from 0 to 1, with higher values indicating better classification performance. Test accuracy should be close to training accuracy; large differences suggest overfitting.
            \item \code{auc_train/auc_test}: Area Under the Curve measures the model's ability to distinguish between classes. Values range from 0.5 (random) to 1.0 (perfect discrimination). AUC > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test AUC should be close to training AUC to avoid overfitting.
            \item \code{log_loss_train/log_loss_test}: Log Loss (logarithmic loss) penalizes confident wrong predictions more heavily. Lower values indicate better model performance. Values close to 0 are ideal. Test log loss should be close to training log loss; higher test log loss suggests overfitting.
          }
        \item \strong{For Cox regression:}
          \itemize{
            \item \code{c-index_train/c-index_test}: Concordance Index (C-index) measures the model's ability to correctly rank survival times. Values range from 0.5 (random) to 1.0 (perfect ranking). C-index > 0.7 is considered acceptable, > 0.8 is good, > 0.9 is excellent. Test C-index should be close to training C-index to avoid overfitting.
            \item \code{auc_hc}: Harrell's C-index for time-dependent AUC, measuring discrimination at specific time points. Higher values indicate better discrimination ability.
            \item \code{auc_uno}: Uno's C-index for time-dependent AUC, providing an alternative measure of discrimination that may be more robust to censoring patterns.
            \item \code{auc_sh}: Schemper and Henderson's C-index for time-dependent AUC, offering another perspective on model discrimination performance.
          }
      }
    \item \code{overview} A nested list organized by strategy and metric, containing step-by-step summaries of the model-building process. Each element shows which variables were entered or removed at each step along with the corresponding metric values (e.g., AIC, BIC, SBC).
    \item \code{detail} A nested list organized by strategy and metric, providing granular information about each candidate step. This includes which variables were tested, their evaluation statistics, p-values, and whether they were ultimately selected or rejected.
    \item \code{fitted model object within the strategy-specific list} A nested list object organized with a first layer representing the selection strategy (e.g., forward, backward, bidirection, subset) and a second layer representing the metric (e.g., AIC, BIC, SBC). For each strategy-metric combination, the function returns fitted model objects that can be further analyzed using S3 generic functions such as \code{summary()}, \code{anova()}, or \code{coefficients()}. These functions adapt to the model type (e.g., \code{coxph}, \code{lm}, \code{glm}) through call-specific methods. Specific statistics can be directly retrieved using the \code{$} operator, such as \code{result$forward$AIC$coefficients}. The level of detail in these analyses depends on the model type: the \CRANpkg{survival} package enriches \code{coxph} objects with detailed statistics including hazard ratios, standard errors, z-statistics, p-values, and likelihood ratio tests, while base R functions like \code{lm} and \code{glm} offer basic output with coefficients by default, requiring \code{summary()} or \code{anova()} to reveal standard errors, t-values, p-values, and R-squared values.
  }
}
\description{
Performs stepwise regression model selection using various strategies and selection criteria.
Supports multiple regression types including linear, logistic, Cox, Poisson, and Gamma regression.
}
\examples{
# Multivariate linear regression with bidirectional selection
data(mtcars)
formula <- cbind(mpg, drat) ~ . + 0
result1 <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = "bidirection",
  metric = "AIC"
)

summary(result1$bidirection$AIC)
anova(result1$bidirection$AIC)
coefficients(result1$bidirection$AIC)

# Linear regression with multiple strategies and metrics
formula <- mpg ~ . + 1
result2 <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = c("forward", "bidirection"),
  metric = c("AIC", "SBC", "SL", "AICc", "BIC", "HQ")
)

summary(result2$forward$AIC)
anova(result2$forward$AIC)
coefficients(result2$forward$AIC)

# Logistic regression with significance level criteria
data(remission)
formula <- remiss ~ .
result3 <- stepwise(
  formula = formula,
  data = remission,
  type = "logit",
  strategy = "forward",
  metric = "SL",
  sle = 0.05,
  sls = 0.05
)

summary(result3$forward$SL)
anova(result3$forward$SL)
coefficients(result3$forward$SL)

# Linear regression with continuous-nested-within-class effects
mtcars$am <- factor(mtcars$am)
formula <- mpg ~ am + cyl + wt:am + disp:am + hp:am
result4 <- stepwise(
  formula = formula,
  data = mtcars,
  type = "linear",
  strategy = "bidirection",
  metric = "AIC"
)

summary(result4$bidirection$AIC)
anova(result4$bidirection$AIC)
coefficients(result4$bidirection$AIC)

}
\references{
\itemize{
  \item Alsubaihi et al. (2002) Variable strategy in multivariable regression using sas/iml
  \item Darlington (1968) Multiple regression in psychological research and practice
  \item Dharmawansa et al. (2014) Roy's largest root under rank-one alternatives
  \item Hannan & Quinn (1979) The determination of the order of an autoregression
  \item Hotelling (1992) The Generalization of Student's Ratio
  \item Hocking (1976) The analysis and strategy of variables in linear regression
  \item Hurvich & Tsai (1989) Regression and time series model strategy in small samples
  \item Judge (1985) The Theory and practice of econometrics
  \item Mallows (1973) Some comments on cp
  \item Mardia et al. (1979) Multivariate analysis
  \item Mckeon (1974) F approximations to the distribution of hotelling's t20
  \item Mcquarrie & Tsai (1998) Regression and Time Series Model strategy
  \item Pillai (1955) Some new test criteria in multivariate analysis
  \item Sparks et al. (1985) On variable strategy in multivariate regression
  \item Sawa (1978) Information criteria for discriminating among alternative regression models
  \item Schwarz (1978) Estimating the dimension of a model
}
}
\author{
Junhui Li, Kai Hu, Xiaohuan Lu
}
\keyword{regression}
\keyword{stepwise}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{report}
\alias{report}
\title{Generate Stepwise Regression Report}
\usage{
report(x, report_name, format = c("html", "docx", "rtf", "pptx"))
}
\arguments{
\item{x}{A StepReg object containing the results of stepwise regression analysis.}

\item{report_name}{Character. The name of the output report file(s) without extension.}

\item{format}{Character vector. The output format(s) for the report. Choose from:
\itemize{
  \item "html" - Web page format (default)
  \item "docx" - Microsoft Word document
  \item "pptx" - Microsoft PowerPoint presentation
  \item "rtf" - Rich Text Format
}
Multiple formats can be specified simultaneously.}
}
\value{
Creates report file(s) in the specified format(s) in the current working directory.
The file name will be \code{report_name.format} (e.g., "myreport.html", "myreport.docx").
}
\description{
Creates formatted reports from StepReg objects in various document formats. This function
generates comprehensive reports containing all tables and results from the stepwise regression
analysis.
}
\details{
The generated report includes:
  \itemize{
    \item Summary of model parameters and selection criteria
    \item Variable types and classifications
    \item Step-by-step selection process
    \item Final selected model and fit statistics
    \item Model coefficients and significance levels
  }
  
  \strong{Note:} Exporting reports requires pandoc to be installed on your system. 
  Pandoc is typically included with RStudio. If not available, install it from 
  https://pandoc.org/installing.html or via your system package manager.
}
\examples{
\dontrun{
# Load leukemia remission data
data(remission)

# Run stepwise logistic regression
formula <- remiss ~ .
result <- stepwise(
  formula = formula,
  data = remission,
  type = "logit",
  strategy = c("forward", "bidirection"),
  metric = c("AIC", "BIC")
)

# Generate reports in multiple formats
report(
  x = result,
  report_name = "leukemia_analysis",
  format = c("html", "docx")
)
}

}
\seealso{
\code{\link{stepwise}} for creating StepReg objects

\code{\link{plot.StepReg}} for visualization of results
}
